# PC端--新手向导
>用于前端指令支持后端插件调用，去介绍的页面的新组件或者是原有组件的新功能。


## 视觉展示

- **向导单步骤条** 

![default.png](./image/image1.png) 

- **向导多步骤条** 

![playing.png](./image/image3.png) 

> 由上图可以看到`单步骤向导`和`多步骤向导`最明显的区别，则是`多步骤向导`有`下一步`和`完成`按钮,`多步骤向导`需要通过点击下一步直到`完成`按钮，`单步骤向导`则只有一项，点击图片即可关闭


## 实例指令 

```Java
//多步骤条实例
{"p":[{"guideItems":[{"offset":{"top":-15,"left":-324},"extendPic":"","tip":{"offset":{"top":14,"left":0},"arrowOffset":309,"step":"1","title":"个人中心","icourl":"iconUrl","content":"个人通知、切换皮肤、切换组织"},"pageId":"pageId","key":"usericon","showPos":1},…]}],"a":"showGuide"}
```
```Java
//单步骤条实例
{"p":[{"guideItems":[{"offset":{"top":-15,"left":-324},"extendPic":"","tip":{"offset":{"top":14,"left":0},"arrowOffset":309,"step":"1","title":"个人中心","icourl":"iconUrl","content":"个人通知、切换皮肤、切换组织"},"pageId":"pageId","key":"usericon","showPos":1},…]}],"a":"showGuide"}
```
属性名 | 说明 | 类型 | 说明
------- | ------- | ------- |------
guideItems| 向导步骤的具体显示内容 | `Object|Array` | (`单步骤条与多步骤的区分，就在这了传入的是数组还是对象，数组则是多步骤向导，对象则是单步骤条`)
offset | 跳出的向导于控件的相对（介绍的控件）高亮元素偏移位置 | `Object`  | 无
tip | 向导框里面的具体内容 | `Object` | 无
key | 需要展示高亮的控件 | `string` | 无
showPos | 显示的相对位置 | `number` | 0:up 1:bottom 2:right 3：left
extendPic|	模拟控件弹出面板效果图片链接| `string` | 无

## tip中属性名介绍

属性名 | 说明 | 类型 | 说明
------- | ------- | ------- |------
offset | 向导内容相对于向导框的偏移 | `Object`  | 无
arrowOffset | 指示箭头相对于向导框的偏移 | `Object` | 需要和`showPos`搭配使用，如是`showpos:1`则箭头则会显示在上边框，偏移则是在上边框偏移
step | "显示哪些步数操作(比如上一步,下一步,跳过)" | `string` | 无
title | 向导内容的title | `string` | 无
icourl | 内容的Icon图片路径 | `string` | 无
content | 向导内容 | `string` | 无


## 功能说明

> PC端向导是为了提醒和介绍PC端页面新的组件或者是原有组件的新功能，可以展示介绍任何页面的的任何控件，目前只是介绍，没法做到引导用户操作



## 插件示例

- **插件代码**


```Java
// 通知前端展示首页向导
// 获取首页向导数据
Map<String, Object> guideItems = getHomepageGuideData();
proxy.addAction("showGuide", guideItems);
		
```

```Java
//构造数据
    /**
	 *
	 * @param icourl 	图标链接地址
	 * @param title  	标题
	 * @param content	具体描述内容
	 * @param step		显示哪些步数操作(比如上一步,下一步,跳过)
	 * @param tiptop	信息框显示位置的偏移
	 * @param tipleft	信息框显示位置的偏移
	 * @param pageId	要高亮的控件所在页面的
	 * @param key		页面上要高亮的控件id
	 * @param showPos	显示位置(上下左右居中)01234
	 * @param extendPic	模拟控件弹出面板效果图片链接
	 * @param alltop 	整个信息框的偏移
	 * @param allleft	整个信息框的偏移
	 * @param arrowOffset 三角箭头偏移量
	 * @return
	 */
private Map<String,Object> getHomepageGuideData() {
		//获取每一列向导的数据
		Map<String,Object> guideItemsData = new HashMap<>();
		List<Map<String,Object>> guideItems = new ArrayList<> ();
		Map<String,Object> guideItem = getGuideItemData(
 				"展示图片路径1","新手第一页Title",
 				"新手第一页Context","stepNums","tiptop","tipleft","pageId","key","showPos",
				"",
				"alltop","allleft","arrowOffset");
		guideItems.add(guideItem);
		Map<String,Object> guideItemSecond = getGuideItemData(
 				"展示图片路径2","新手第二页Title", 
 				"新手第二页Context","stepNums","tiptop","tipleft","pageId","key","showPos",
				"",
				"alltop","allleft","arrowOffset");
		guideItems.add(guideItemSecond);
		Map<String,Object> guideItemThird = getGuideItemData(
 				"展示图片路径3","新手第三页Title",
 				"新手第三页Context","stepNums","tiptop","tipleft","pageId","key","showPos",
				"",
				"alltop","allleft","arrowOffset");
		guideItems.add(guideItemThird);
		guideItemsData.put("guideItems",guideItems);
		return guideItemsData;
    }
    

    private Map<String, Object> getGuideItemData(String icourl, String title, String content, String step,
			int tiptop, int tipleft, String pageId, String key, int showPos, String extendPic,
			int alltop, int allleft,int arrowOffset) {
		Map<String,Object> guideItem =  new HashMap<>();
		Map<String,Object> tip = new HashMap<>();
		tip.put("icourl", icourl);
		tip.put("title", title);
		tip.put("content", content);
		tip.put("step", step);
		Map<String,Object> offset = new HashMap<>();
		offset.put("top",tiptop);
		offset.put("left", tipleft);
		tip.put("offset", offset);
		tip.put("arrowOffset", arrowOffset);
		guideItem.put("tip", tip);
		guideItem.put("pageId",pageId);
		guideItem.put("key", key);
		guideItem.put("showPos",showPos);
		guideItem.put("extendPic",extendPic);
		Map<String,Object> alloffset = new HashMap<>();
		alloffset.put("top",alltop);
		alloffset.put("left", allleft);
		guideItem.put("offset",alloffset);
		return guideItem;
	}
```



