# 如何异步处理耗时操作
在苍穹中一切前后端的交互都是指令，用户操作界面=>前端发送请求=>后端插件处理业务逻辑后返回指令=>前端响应返回的指令，结束该一交互过程。 

要想前后端连续交互，苍穹页提供了多种方式：  
1、使用进度条控件化身为定时器控件；  
2、`addClientCallBack`指令  

该章节重点描述addClientCallBack如何使用  

?> **场景：打开首页后，个人头像显示未读消息数，获取个人未读消息比较耗时。**  

**原处理**
```  java
@Override
public void afterCreateNewData(EventObject e) {
    /**
        其他首页处理逻辑
    */

    // 待办任务数量
    long toHandleTaskCount = WorkflowServiceHelper.getTaskCountByType(userIdStr, WorkflowServiceHelper.TOHANDLE);
    // 未读消息数量
    long toHandleMsgCount = MessageCenterServiceHelper.getUnReadMessageCount(userIdStr);
    long total = toHandleTaskCount + toHandleMsgCount;
    if (total > 0) {
        Map<String, Object> imgParamMap = new HashMap<>();
        imgParamMap.put("count", total);
        imgParamMap.put("offset", new Integer[] { 0, -5 });
        imgParamMap.put("overflowCount", 99);
        proxy.invokeControlMethod("usericon", "setBadgeInfo", imgParamMap);
    }
}
```

**使用addClientCallBack优化后**  
```  java
@Override
public void afterCreateNewData(EventObject e) {
    /**
        其他首页处理逻辑
    */
    
   this.getView().addClientCallBack("personalInfo", 100);
}

/**
    addClientCallBack到期回调方法
*/
@Override
public void clientCallBack(ClientCallBackEvent e) {
    if("personalInfo".equals(e.getName())) {
        String userIdStr = RequestContext.get().getUserId();
         // 待办任务数量
        long toHandleTaskCount = WorkflowServiceHelper.getTaskCountByType(userIdStr, WorkflowServiceHelper.TOHANDLE);
        // 未读消息数量
        long toHandleMsgCount = MessageCenterServiceHelper.getUnReadMessageCount(userIdStr);
        long total = toHandleTaskCount + toHandleMsgCount;
        if (total > 0) {
            Map<String, Object> imgParamMap = new HashMap<>();
            imgParamMap.put("count", total);
            imgParamMap.put("offset", new Integer[] { 0, -5 });
            imgParamMap.put("overflowCount", 99);
            proxy.invokeControlMethod("usericon", "setBadgeInfo", imgParamMap);
        }
    }
}
```
!>优化结果：原来首页加载前同时去获取了个人未读消息，影响了首页的加载时间，优化后，afterCreateNewData中不处理该逻辑，指返回了addClientCallBack指令，待首页加载后的100ms，前端自动回调clientCallBack方法，此时在处理个人未读信息，把原来一个请求的逻辑，合理的拆分未两个请求，达到优化效果。

**this.getView().addClientCallBack(String actioin,Integer delayTime)**

actioin：为标识，用于在回调中识别  
delayTime：为回调时间，单位ms，如前端收到该指令后100ms回调

**clientCallBack(ClientCallBackEvent e)**

为后端回调方法，在该方法处理回调后的逻辑