## 起步
该章节将介绍如何开发一个自定义控件，通过一个最简单的例子`hello world`的开发，你将会掌握开发自定义控件的每一个步骤。下面跟着我进入自定义控件的开发吧！！

- **新建一个`helloworld`目录， 在该目录下新建`index.js`文件：**

![createdir](./image/createdir.png ':size=500')

- **使用任意编辑器打开`index.js`文件，复制下面代码（可以先忽略代码逻辑，后面会描述）到该文件中：**

```javascript
/**
 *  自定义控件书写模板
 */
(function(KDApi, $){
    // 构造函数，变量名随意，与最后一句代码的KDApi.register的第二个参数一致即可
    function HelloWorld (model) {
        this._setModel(model)
    }

    // 原型中封装生命周期函数，固定格式
    HelloWorld.prototype = {
        _setModel: function(model) {
            this.model = model
        },
        init: function(props){
            // TO DO
            initFunc(this.model, props)
        },
        update: function(props){
            // TO DO
        },
        destoryed: function(){
            // TO DO
        }
    }

    // Other Code
    var initFunc = function(model, props) {
        model.dom.innerHTML = '<h1>init: Hello World!</h1>'
    }

    // 你只需知道第一个参数是你接下来要新增控件方案时填的方案id，第二个参数是上面声明的构造函数
    KDApi.register('helloworld', HelloWorld)
})(window.KDApi, jQuery) // 这里的jQuery不是必须要传进去的，可移除，要用到的时候才传，PC端系统默认会有jQuery对象，版本是1.12.4
```

!> 以上模板适用于PC端的自定义控件开发，如果是移动端的自定义控件开发，则需要将模板中使用的`jQuery`对象去除，移动端不提供`jQuery`对象。假如你一定要在移动端使用`jQuery`，你可以狠狠地[戳下这里](./advanced/thridJs/)，了解`扩展JS`的使用

- **将`helloworld`目录下的文件打成zip包，注意`不要将helloworld文件夹也打包了，只要其目录下的文件`，且zip包下根目录一定要有`index.js`文件**

好了，到此我们该控件前端编码部分已经完成，我们有一个`helloworld.zip`文件，它该如何使用呢？

- **在设计器上找到`自定义控件`，添加到你的页面中，并设置其标识为`helloworld`，点击`控件方案`，新增控件方案，方案id为helloworld，领域标识为cq，并且选择上传helloworld.zip**

![newandupload](./image/newandupload.png ':size=600')

- **点击`保存`，选择这条方案，点击`返回数据`，在设计器上点击`保存`，并`预览`**

![savepre](./image/savepre.gif ':size=300')

Good!!!，我们的第一个例子hello world就到此完成了！

?> 回顾上面例子，我们完成一个自定义控件需要以下几个步骤：  
1、建一个包含index.js文件的目录，index.js是程序入口；  
2、按照模版代码编辑前端js逻辑，完成后打包成zip；  
3、设计器上新增控件方案，上传zip包，方案和zip关联；   
4、设计器上新增自定义控件，自定义控件选择该方案；  

!> 在上面的例子中，我们逻辑只有`model.dom.innerHTML = '<h1>init: Hello World!</h1>'`，就是在页面初始化的时候输出一句话，如果我们的控件很复杂时，我们还是通过innerHTML的方式拼接我们的页面吗？我们使用html和css写复杂交互的控件，同时让js和html和css分离？答案是：当然可以！！

## CSS、HTML、JS分离

当开发复杂的自定义控件时，建议css、html、css分离处理，方便开发和维护，我们默认集成了[art-template](https://aui.github.io/art-template/zh-cn/docs/syntax.html)字符串模板，你可以直接该语法编写html（非强制），并可通过`KDApi.getTemplateStringByFilePath`接口[(更多接口信息)](./ctrl/commonControl/customctrl/?id=前端js-api)解析为html字符串，接下来让我们继续完善之前的例子。

- **在`helloworld`文件夹下，新建`html`目录并在该目录下，新建helloworld.html文件**

- **将一下代码复制粘贴到`helloworld.html`中**

```html
<h1>init: <%= text %>!</h1>
```

- **修改`index.js`里的`initFunc`函数的代码，使用以下代码，通过模版的方式分离html**

```javascript
var initFunc = function(model, props) {
    // 通过路径去获取html字符串，第一个参数是模版文件路径，第二个参数是model，第三个参数是HTML模板中变量的值
    KDApi.getTemplateStringByFilePath('./html/helloworld.html', model, {
        text: 'Hello, World!'
    }).then(function(result) {
        model.dom.innerHTML = result
    })
}
```

我们接下来给该控件添加css

- **修改`helloworld.html`，添加一个css**

```html
<h1 class="helloworld">init: <%= text %>!</h1>
```

- **在`helloworld`文件夹下，新建`css`目录并在该目录下，新建helloworld.css文件，并在`helloworld.css`中添加一个helloworld样式**

```css
.helloworld{
    color: #C0C0C0; //修改字体颜色
    border: 1px solid #C0C0C0; //添加边框
}
```

!> 建议css的class或者id的命名，遵循规范：使用`.领域_schemaId_`为前缀，防止与其他class或者id出现冲突。禁止直接使用标签选择器，例如：a {...}：禁止，.hr_dellabel a{...}：允许。

- **修改`index.js`里的`initFunc`函数，使用`KDApi.loadFile`把css加载进来，代码如下**

```javascript
var initFunc = function(model, props) {
    // KDApi.loadFile可以通过路径加载js或css文件，并且在html文件头生成script或者link标签，第一个参数是路径，第二个参数是model，第三个参数是加载完成后执行的回调函数
    KDApi.loadFile('./css/helloworld.css', model, function() {
        // 通过路径去获取html字符串，第一个参数是路径，第二个参数是model，第三个参数是HTML模板中变量的值
        KDApi.getTemplateStringByFilePath('./html/helloworld.html', model, {
            text: 'Hello, World!'
        }).then(function(result) {
            model.dom.innerHTML = result
        })        
    })
}
```

- **`helloworld`目录下的文件重新打包成helloworld.zip并上传，保存预览后，效果如下：**

![addcss](./image/addcss.png ':size=300')

至此，我们就完成简单的css、html、js分离啦！

?> 回顾上面例子，我们完成了自定义控件的html、css、JS的分离写法：  
1、通过KDApi.getTemplateStringByFilePath方法把art-template模版引入并解析；  
2、通过KDApi.loadFile方法把css文件引入；    
 
!> 在上面的例子中，我们自定义控件并没有和后端插件有数据交互，数据都是在js里面写死的，那后端插件的数据是如何发送到该自定义控件呢，我的插件该如何写呢？接下来为你介绍。

## 后端插件为自定义控件赋值

日常我们可以使用java插件或者KDE脚本插件，给自定义控件传输业务数据。该例子使用KDE脚本插件，java插件的写法和KDE脚本大同小异。

- **新建脚本插件`helloworld.ks`，并且将下面代码复制到插件中**

```javascript
var plugin = new FormPlugin({
	afterBindData : function(e){
        // getControl中传入的是控件的标识（key）不是方案id（schemaId）
		var customcontrol = this.getView().getControl('helloworld');
        var data = {
            text: 'Hello World!'
        }
        //调用该自定义控件的setData方法赋值
        customcontrol.setData(data)
	}
});
```

- **修改`index.js`里的`initFunc`函数，我们可以从props中获取得到控件的data，更多关于props信息点击该[链接](./ctrl/commonControl/customctrl/?id=数据（props）)**

```javascript
var initFunc = function(model, props) {
    // KDApi.loadFile可以通过路径加载js或css文件，并且在html文件头生成script或者link标签，第一个参数是路径，第二个参数是model，第三个参数是加载完成后执行的回调函数
    KDApi.loadFile('./css/helloworld.css', model, function() {
        // 后端插件通过setData传给前端的数据，前端可以通过props.data去获取
        var text = props.data && props.data.text || ''
        // 通过路径去获取html字符串，第一个参数是路径，第二个参数是model，第三个参数是HTML模板中变量的值
        KDApi.getTemplateStringByFilePath('./html/helloworld.html', model, {
            text: text
        }).then(function(result) {
            model.dom.innerHTML = result
        })        
    })
}
```

?> 回顾上面例子，我们完成了自定义控件插件赋值的方法：  
1、通过在插件中调用控件的setData方法，该方法接收任意类型的数据，因此开发自定义控件前应该设计好你的控件数据结构；     
 
!> 在上面的例子中，我们自定义控件有了后端插件赋值能力，解决了我们纯展示类的控件需求，但如果我们控件更加复杂，有点击，有编辑等需求，控件要和后端插件实现主动通讯，我的控件和插件该如何写呢？接下来为你介绍。

## 自定义控件前后端完整交互

在这一小节里，我们会通过前端的点击事件给后端发请求，后端插件接收到这个请求后弹出一个提示框，这样即可完成我们的前后端完整的交互。

- **在`index.js`里，声明`initEvent`函数，监听点击事件，并执行`·model.invoke`方法给后端发送请求[更多关于model](./ctrl/commonControl/customctrl/?id=模型（model）)，代码如下**

```javascript
var initEvent = function(model, props){
    //内置了jquery对象，可直接使用$
    $('.helloworld', model.dom).click(function(){
        // model.invoke，用于给后端发送请求，第一个参数是事件名，可自定义；第二个参数是发送给后端的数据，可以是任意类型
        model.invoke('click', 'Hello World!')
    })
}
```

- **在`index.js`里的`initFunc`函数中，通过initEvent给DOM绑定操作事件**

```javascript
var initFunc = function(model, props) {
    // KDApi.loadFile可以通过路径加载js或css文件，并且在html文件头生成script或者link标签，第一个参数是路径，第二个参数是model，第三个参数是加载完成后执行的回调函数
    KDApi.loadFile('./css/helloworld.css', model, function() {
        // 后端插件通过setData传给前端的数据，前端可以通过props.data去获取
        var text = props.data && props.data.text || ''
        // 通过路径去获取html字符串，第一个参数是路径，第二个参数是model，第三个参数是HTML模板中变量的值
        KDApi.getTemplateStringByFilePath('./html/helloworld.html', model, {
            text: text
        }).then(function(result) {
            model.dom.innerHTML = result
            // 绑定DOM事件
            initEvent(model, props)
        })        
    })
}
```

- **在脚本插件`helloworld.ks`的`customEvent`方法里，处理前端发送过来的请求**

```javascript
var plugin = new FormPlugin({
	afterBindData : function(e){
        // getControl中传入的是控件的标识（key）不是方案id（schemaId）
		var customcontrol = this.getView().getControl('helloworld');
        var data = {
            text: 'Hello World!'
        }
        customcontrol.setData(data)
    },
    customEvent: function(e) {
        // 设计器上自定义控件的标识
        var key = e.getKey()
        // 前端通过model.invoke传给后端的数据
        var args = e.getEventArgs()
        // 前端通过model.invoke定义的事件名
        var eventName = e.getEventName()
        this.getView().showMessage('key: ' + key + ';eventName: ' + eventName + ';args: ' + args)
    }
});
```
保存后预览，效果如下：

![click](./image/click.gif)

至此，我们完整的前后端交互的例子完成了！

?> 回顾上面例子，我们完成了自定义控件前后端交互的方法：  
1、控件中通过调用model.invoke方法，给插件发送请求；  
2、在插件中的customEvent接收自定义控件发送过来的请求，并做处逻辑处理；

## 生命周期

每个自定义控件有自己的生命周期，分别是`构造（_setModel）`、`初始化（init）`、`更新（update）`、`销毁（destoryed）`。下面将具体介绍每个生命周期的使用：

**`_setModel`将元数据和业务数据传给自定义控件，可忽略；**  
**`init`初始化阶段，自定义控件初次渲染在页面上，可以在此方法加载资源，请求数据等；**     
**`update`更新阶段，当业务数据或元数据发生改变时，会调用该方法，如插件调用控件setData后，修改了该控件的值；**    
**`destoryed` 页面关闭销毁时，自定义控件也会销毁，常用于取消事件监听或释放资源等操作。**   

上一小节的例子中只涉及到了自定义控件的初始化，接下来我们来聊聊自定义控件的更新和销毁。

- **在KDE脚本插件里，在`customEvent`中通过setData传数据给自定义控件**

```javascript
var plugin = new FormPlugin({
	afterBindData : function(e){
        // getControl中传入的是控件的标识（key）不是方案id（schemaId）
		var customcontrol = this.getView().getControl('helloworld');
        var data = {
            text: 'Hello World!'
        }
        customcontrol.setData(data)
    },
    customEvent: function(e) {
        // 设计器上自定义控件的标识
        var key = e.getKey()
        // 前端通过model.invoke传给后端的数据
        var args = e.getEventArgs()
        // 前端通过model.invoke定义的事件名
        var eventName = e.getEventName()
        this.getView().showMessage('key: ' + key + ';eventName: ' + eventName + ';args: ' + args)

        // 响应前端点击事件，将新的数据传给自定义控件，把init: Hello World!改为update: Hello World!
        if (eventName === 'click') {
            var customcontrol = this.getView().getControl('helloworld')
            var updateData = {
                text: 'update: Hello World!'
            }
            customcontrol.setData(updateData)            
        }
    }
});
```

- **在`index.js`里，声明`updateFunc`函数，并在`update`周期函数里面调用**

```javascript
(function(KDApi, $){
    function HelloWorld (model) {
        this._setModel(model)
    }

    HelloWorld.prototype = {
        _setModel: function(model) {
            this.model = model
        },
        init: function(props){
            // TO DO
            initFunc(this.model, props)
        },
        update: function(props){
            // TO DO
            updateFunc(this.model, props)
        },
        destoryed: function(){
            // TO DO
        }
    }

    // Other Code

    // initFunc...
    // ...
    // ...

    var updateFunc = function(model, props) {
        var updateText = props.data && props.data.text || ''
        $('.helloworld', model.dom).text(updateText)
    }

    // initEvent...
    // ...
    // ...

    KDApi.register('helloworld', HelloWorld)
})(window.KDApi, jQuery)
```

- **在`index.js`里的destoryed周期函数里，做弹框提示**

```javascript
HelloWorld.prototype = {
    _setModel: function(model) {
        this.model = model
    },
    init: function(props){
        // TO DO
        initFunc(this.model, props)
    },
    update: function(props){
        // TO DO
        updateFunc(this.model, props)
    },
    destoryed: function(){
        // TO DO
        alert('destory: Hello World!')
    }
}
```

重新打包上传，保存后预览，效果如下：

![circle](./image/circle.gif)

至此，我们算是走完了自定义控件整个生命周期流程！

## KDApi.register的作用

!> 这个接口的作用就是注册helloworld自定义控件，这句代码一定要写！一定要写！一定要写！

- **KDApi.register('helloworld', HelloWorld)**

第一个参数是控件方案id，在新增控件方案的时候填写的控件方案id是一样的；第二个参数是闭包里声明的工厂函数，在上述例子里是HelloWord。

?> 你可以狠狠地[戳下这里](./advanced/customcontrol/example/helloworld.zip ':ignore')下载上述完整的`helloworld`例子

?> 更多关于自定义控件的`model`、`props`及相关的`API`，你可以[点击这里](./ctrl/commonControl/customctrl/)查看