# 如何获取移动端定位信息

> **获取移动端定位信息，可用于展示用户当前所在城市等操作，过程分为两步：1、调用获取当前位置指令；2、插件接收当前位置信息**

## 调用获取当前位置指令

获取当前位置信息的方法有两种：  
- `设计器属性配置`  
- `插件指令主动调用`  
  
针对不同的场景可选择不同的调用方式。

### 设计器属性配置

![mobile-bill-location](./image/mobile-bill-location.jpg ':size=650x450')

在设计器中启用移动单据的属性'启用定位'，页面打开时会自动将定位信息发送到后端(`具体操作如上截图`)

操作步骤：
> * 步骤1：
打开对应移动单据，选中`单据根节点`
> * 步骤2：
在单据属性面板里，启用属性`启用定位`

前端向后端发送的定位信息数据示例

```javascript
{"city":"深圳市"}

//定位发送后端信息是`city`城市信息
```

### 插件指令主动调用
> 通过后端插件主动调用`getLocation`指令 , 动态获取页面定位信息.

后端插件调用代码示例：
```java
public void locate() {
    //调用getLocation指令获取客户端定位信息
    IClientViewProxy clientViewProxy = this.getService(IClientViewProxy.class);
	clientViewProxy.addAction(ClientActions.getLocation, new Object[0]);
}
```

前端向后端发送的定位信息数据示例

```javascript
{
    "latitude": 22.537049799521217,
    "longitude": 113.95124389999997,
    "province": "广东省",
    "city": "深圳市",
    "district": "南山区",
    "name": "",
    "address": "香港中文大学深圳研究院大楼内,BWE POINT LAFE附近25米",
    "addressdetail": "广东省深圳市南山区粤兴二道10 香港中文大学深圳研究院大楼内,BWE POINT LAFE附近25米"
}

//定位发送到后端的是详细定位信息，包含省市、经纬度等. 通过页面请求'setLocation'实现
```

## 插件接收当前位置信息

后端插件调用代码示例：
```java

//方法1: 插件中重写'locate'方法，当前端(客户端)回发定位信息到服务端，会自动触发这里的回调，获取到定位信息
@Override
public void locate(LocateEvent e) {
    super.locate(e);
    MobLocation location = e.getMobLocation(); //MobLocation对象包含的就是定位信息
}

//方法2:直接从服务端缓存的定位信息里获取
public void getLocationInfo() {
    //获取具体定位信息
    MobileBillView view = (MobileBillView)this.getView();
    MobLocation location1 = view.getLocation(); //MobLocation对象包含的就是定位信息
}
```