# 基础资料
>封装了一组基础数据的字段，每条基础数据含有多个属性

## 控件对象
`kd.bos.form.field.BasedataEdit`

## 视觉
- **垂直**

![垂直](./image/style_v.png)
- **水平**

![水平](./image/style_h.png)
- **模糊搜索**

![模糊搜索](./image/search.gif)

- **下拉展示数据**

![下拉风格](./image/dropdown.png)

## 属性说明

### 字段通用属性

?> 字段通用属性[请参考](ctrl/commonField/text/)

### 业务属性

属性名 | 属性ID | 类型 | 默认值 | 说明
:----: | :----: | :----: | :----: | ----
快捷新增 | | Boolean | true | 基础资料模糊查询无记录时显示快捷新增按钮，可快速新增基础资料数据
基础资料类型 | `ei` | String | | 配置关联的基础资料，<u>必须配置</u>
使用组织  | | | | 关联对应的`组织`基础资料
编辑显示属性 | `ep` | String | `number` | 设置在编辑时基础资料的属性：编码（`number`）、名称（`name`）、编码+名称(`number,name`)、名称+编码(`name,number`)
显示属性 | `dsp` | String | `name` | 设置基础资料显示的内容:编码（`number`）、名称（`name`）、编码+名称(`number,name`)、名称+编码(`name,number`)
引用属性 | | | | 设置引用对应自己资料的属性，可以引用多个
过滤条件 | | | | 配置基础资料对应的过滤条件
是否只显示启用 | | Boolean | `true` | 配置F7只显示`启用`的数据，默认`开启`
显示风格 | `dst` | Number | `0` |  配置基础资料展示数据样式：缺省（`0`）、下拉（`1`）
缺省值  | | | | 配置基础资料默认值
查看详情 | | Boolean | `true` | 设置查看详情
支持模糊查询 | | Boolean | `true` | 勾选后开启模糊查询功能，输入时可以匹配基础资料数据
启用我的常用 | `sft` | Boolean | `false` | 开启常用功能
为空提示信息 | `emptytip` | Map | | 当字段内容为空时，输入框内显示的信息
编辑风格 | `editStyle` | Number | | 编辑风格有两种：按钮+文本编辑和按钮点击编辑(`1`), 默认是按钮+文本编辑模式
按钮图标 | `fontClass` | string | | 基础资料后面的按钮图标

### 样式属性

?> 样式属性[请参考](ctrl/commonField/text/)

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例

```java
public class TestPlugin extends AbstractFormPlugin implements BeforeF7SelectListener, BasedataFuzzySearchListener {
  public void registerListener(EventObject event) {
    BasedataEdit edit = this.getView().getControl("基础资料标识");
    edit.addBeforeF7SelectListener(this);
    edit.addBasedataFuzzySearchListener(this);
  }
  
  // F7选择数据前进入该方法
  public void beforeF7Select (BeforeF7SelectEvent evt) {
  
  }
  
  // 模糊查询结果处理方法
  public void handleData (BasedataFuzzySearchEvent evt) {
  
  }

}
```  


## 事件监听接口
监听接口 | 方法名 | 说明  
------- | ------- | -------  
`BeforeF7SelectListener` | `beforeF7Select` |F7选择前触发该监听  
`BasedataFuzzySearchListener` |`handleData` | 基础资料模糊查询监听

## 插件API

方法名 | 参数 | 返回值 | 说明  
------- | ------- | ------- | ------ 
`addBeforeF7SelectListener`|`BeforeF7SelectListener`|无|增加F7选择前监听接口
`addBasedataFuzzySearchListener`|`BasedataFuzzySearchListener`|无|增加基础资料模糊查询监听
`getPrimaryKeyProp` | `EntityType`实体类型 | `IDataEntityProperty`|  获取主键属性对象
`getPkIds` | 无 | `Object[]`| 获取所有PK Id
`getMainEntiryType` | 无 |`BasedataEntityType`|获取主实体类型
`getBaseEntityId`|无|`String`|获取基础资料实体Id
`setItemByNumber`|`String`|无|通过id设置选择项
`isEntryProperty`|`IDataEntityProperty`|`boolean`|判断属性所在实体是否分录
`getCurrentRowIndex`|无|`int`|获取当前行号，表头返回`0`

 
