# 卡片分录
> 一个可循环的容器，卡片分录有编辑态与查看态
## 层次结构

![层次结构](./image/cengcijiegou.png)

- **卡片分录视图**：卡片列表层，包含所有卡片
- **卡片**：参与数据循环，有多少条数据就会创建多少张卡片
- **固定卡片**：不参与循环，相当于一个普通的容器，可以在里面放置不需要循环的内容，不需要也可删除，默认隐藏，下图是固定卡片典型应用场景之一。

![固定卡片](./image/gudingcard.png)

## 控件对象
`kd.bos.form.cardentry.CardEntry`

## 属性说明
- **单据体**

属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ----
多选 | `ismul` | boolean| `true` | 设置卡片是否多选  
分页  | `isSplitPage` | boolean | `false` | 设置卡片是否分页  
分页条数 | `pagerows` | number | `20条／页` | 设置每一页显示多少条数据
表名  | | | | 对应数据库的表名
界面规则| | | | 实体服务规则，见下文 `界面规则`
缺省行数 | `defaultrows` | number | 1 | 设置卡片行数
卡片模式 | `cardmode` | number | 0 | 设置卡片模式，0为默认模式，1为[`购物车模式`](#buymode)（目前只支持插件配置），见下文 [`模式说明`](#cardmode)

- **卡片分录视图**

属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ----
布局方式 | `ls` | number | `横向滚动布局` | 设置卡片布局方式，`横向滚动布局`(默认)，只在水平方向延伸，不会换行，超出界面，会出现前后滑动按钮，可以左右滚动。`流式布局`，搭配容器快速布局使用
显示卡片下划线 | `point` | boolean | `false` | 设置卡片是否出现下划线，`卡片单选`、`横向滚动布局` 两个条件同时满足时才生效。
分页隐藏 | `phide` | boolean | `true` | 设置是否隐藏分页工具栏
滚动加载 | `scrollload` | boolean | `false` | 设置是否滚动加载下一页数据

- **卡片**

属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ----
显示选择框 | `showsel` | boolean | `false` | 设置卡片是否显示选中样式
选中样式 | `selectstyle` | number |  | 打开显示选择框时，设置选中样式：默认、勾选（`1`）、边框（`2`）、竖条（`3`）风格
移入移出效果 | `showhov` | boolean | `false` | 设置卡片是否会随着鼠标移动，出现悬停样式
手型光标 | `pointercursor` | boolean | `false` | 设置鼠标移到卡片范围，是否会变成手型光标
背景图片字段 | `bkimgfield` | string | | 设置卡片背景图，关联图片字段
界面规则 | `rules` | string | | 卡片客户端界面规则，见下文 `界面规则`

- **卡片布局容器**

属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ----
可折叠 | `collapsible` | boolean | `false` | 设置布局容器是否折叠
默认折叠 | `defaultcollapse` | boolean | `false` | 在布局容器可折叠时，设置是否默认折叠
折叠摘要 | `cf` | List | | 在布局容器可折叠时，设置折叠时显示的摘要
允许点击 | `clickable` | boolean | `false` | 设置布局容器是否有点击事件
允许全屏 | `maximize` | boolean | `false` | 设置布局容器是否全屏显示
悬停显示元素 | `hsi` | boolean | `false` | 设置卡片在鼠标悬停时显示的内容
悬停隐藏元素 | `hhi` | boolean | `false` | 设置卡片在鼠标悬停时隐藏的内容

?> 控件通用属性[请参考](advanced/controlProperties/)

## 界面规则
可设置一系列规则，达到预置的条件时触发某种动作，经典的应用场景有：`隐藏` `显示` 控件，`锁定` `解锁` 字段。

- **举个例子，显示所有18岁人员的姓名，不是18岁人员姓名隐藏**

1.建立一个`18岁`规则条件。  
2.建一个条件`成立`时的动作。  
3.建一个条件`不成立`时的动作。  

![jiemianguize.gif](./image/jiemianguize.gif)


## <span id="cardmode">模式说明</span>
默认模式点击卡片任一区域都可选中取消，购物车模式类似于淘宝的购物车，只有点击勾选框才可以选中取消。两种模式匹配不同的场景
> 以下是两种模式的示例

- **默认模式**

![默认模式](./image/cardmode0.gif)

- **<span id="buymode">购物车模式</span>**

![购物车模式](./image/cardmode1.gif)


## 插件示例
- **动态创建卡片行**

```java
// 创建一行，返回行号
int rowIndex = this.getModel().createNewEntryRow("单据体标识" );

// 根据数据创建多行，返回行号数组
int count = 5; // 数据行数
int[] rowIndex = this.getModel().batchCreateNewEntryRow("单据体标识", count);

// 根据创建的行，设置内容
for (int i = 0; i < count; i++) {
  this.getModel().setValue("字段标识", 内容, rowIndex[i]);
}
```  

- **删除卡片行**

```java
this.getModel().deleteEntryRow("单据体标识", 行号);

this.getModel().deleteEntryRows("单据体标识", 行号数组);
```  

- **值操作**

```java
this.getModel().getValue("字段标识", 行号);
this.getModel().setValue("字段标识", 内容, 行号);
```  

- **隐藏/锁定 卡片分录某行的某个控件**

```java
CardEntry cardEntry = this.getView().getControl("标识");
// 隐藏
cardEntry.setChildVisible(false, 行号, 控件标识);

// 显示
cardEntry.setChildVisible(true, 行号, 控件标识);

// 锁定
this.getView().setEnable(false, 行号，控件标识);

// 解锁
this.getView().setEnable(true, 行号，控件标识);
``` 

- **下拉刷新事件**

```java
public class TestPlugin extends AbstractFormPlugin implements PullRefreshListener {
  
  public void regiterListener(EventObject e){
    // 1.注册监听
    CardEntry card = this.getView().getControl("控件标识");
    card.addPullRefreshlistener(this);
  }
  
  public void pullRefesh(PullRefreshEvent evt) {
    // 卡片下拉刷新进入
  }
}
``` 

- **设置卡片购物车模式**
```java
Map<String,Object> data = new HashMap<>(2);
data.put("k","卡片单据体标识");
// cardmode设1
data.put("cardmode",1);
IClientViewProxy proxy = this.getView().getService(IClientViewProxy.class);
proxy.addAction(ClientActions.updateControlStates,data)
```