# 向导控件

> 设置向导类型为`页签向导`时用来展示一系列操作的步骤，和页签控件组合使用；当设置向导类型为`常规向导`时多用于显示一个任务或事件的进度，也用于一步步地指引用户完成一系列操作。每个向导步骤项都会随着当前进度改变对应的步骤状态，可以清晰地向用户展示已完成、进行中、未完成和异常状态的步骤。

## 控件对象
`kd.bos.form.control.Wizard`

## 视觉展示
- 默认（横向）（向导类型为`页签向导`）

![默认](./image/scroll_x.png)

- 纵向（向导类型为`页签向导`）

![纵向](./image/scroll_y.png)

- 常规向导

![纵向](./image/steps.gif)


## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ---- 
向导类型 | `wizardType` | String | `tab` | 切换向导控件模式 （`tab`: 页签向导，`normal`: 常规向导）
向导风格 | `wizardStyle` | number | `0` | 显示风格，分为横向和纵向,（`0`： 横向， `1`： 纵向）
  

### 向导类型为`页签向导`时的可配置属性说明

属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ---- 
页签控件 | `tabId` | String | `-` | 配置关联的页签控件，向导步骤的名称和页签的名称一一对应
标题位置 | `textDirection` | number | `0` | 设置标题显示位置（`0`： 上， `1`： 下， `2`：左，`3`：右）

### 向导类型为`常规向导`时的可配置属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ---- 
步骤长度 | `stepsLength` | Number | `4` | 步骤长度，用于快速配置指定长度的步骤条，会被“步骤项配置”属性覆盖（必须为大于0的整数，步骤条方向设置为水平方向时最大4，纵向无限制）
步骤项配置 | `stepsOptions` | Array | `-` | 用于配置步骤条各个步骤项的标题以及描述信息，当该属性有配置时，步骤条实际长度以该属性长度为准
状态图标 | `statusIcon` | Object | `-` | 用于自定义配置各个状态显示的图标（四个状态分别为已完成`finish`、进行中`process`、未完成`wait`和异常`error`）


## 插件示例
- **向导类型为`页签向导`时在插件中调用如下方法可切换向导步骤**

```java
Tab tab = this.getControl("标签控件标识");
tab.activeTab("子页签标识");
```  

- **向导类型为`常规向导`，设置整向导控件的步骤的配置、设置当前步骤及对应的状态**

```java
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.form.control.Wizard;
import kd.bos.form.control.StepsOption;
import kd.bos.form.plugin.AbstractFormPlugin;

import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TestPlugin extends AbstractFormPlugin {

	@Override
	public void afterCreateNewData(EventObject e) {
		Wizard wizard = this.getControl("wizard");

		// 获取设计时的步骤条设置
		List<StepsOption> stepsOptions = wizard.getStepsOptions();
		stepsOptions.clear();

		StepsOption stepsOption0 = new StepsOption();
		stepsOption0.setTitle(new LocaleString("First"));
		stepsOption0.setDescription(new LocaleString("First Step"));
		stepsOption0.setStatus(Steps.FINISH);   // 完成
		stepsOptions.add(stepsOption0);

		StepsOption stepsOption1 = new StepsOption();
		stepsOption1.setTitle(new LocaleString("Second"));
		stepsOption1.setDescription(new LocaleString("Second Step"));
		
		stepsOption1.setStatus(Steps.PROCESS);  // 进行中
		stepsOption1.setStatus(Steps.FINISH);   // 完成
		stepsOption1.setStatus(Steps.ERROR);    // 异常
		stepsOptions.add(stepsOption1);
		
		StepsOption stepsOption2 = new StepsOption();
		stepsOption2.setTitle(new LocaleString("Third"));
		stepsOption2.setDescription(new LocaleString("Third Step"));
		stepsOptions.add(stepsOption2);
		// 更新步骤条设置
		wizard.setClientStepsOptions(stepsOptions);

		// 设置当前节点
		Map<String, Object> currentStepMap = new HashMap<>();
		currentStepMap.put("currentStep", 1);
		currentStepMap.put("currentStatus", Steps.ERROR);
		// 更新当前节点
		wizard.setCurrentStep(currentStepMap);
	}
}
```  

- **向导类型为`常规向导`，监听前端点击向导步骤项发过来的指令**
```java
import kd.bos.form.control.events.WizardStepsListener;

extends AbstractFormPlugin implements WizardStepsListener

	@Override
	public void registerListener(EventObject e) {
		Wizard wizard = this.getControl("wizard");
		wizard.addWizardStepsListener(this);
	}
	
	
	@Override
    public void update(StepEvent e) {
		int value = e.getValue(); // 前端发送的步骤序号
	}
```
