# 树型菜单
>树型导航控件，用于页面导航，每一个菜单都是一个树的根节点，最多包含二级节点。

## 控件对象
`kd.bos.form.control.TreeMenu`

## 视觉展示
![treemenu.jpg](./image/treemenu.gif)

## 属性说明

属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | :---- 
启动菜单收藏 | `collection` | boolean | `true` | 设置菜单是否支持收藏
默认展开 | `ed` | boolean | `false` | 设置菜单是否展开

?> 控件通用属性[请参考](advanced/controlProperties/)

## 菜单配置

- **打开应用的菜单配置**

![menumanager.png](./image/menumanager.png ':size=420')

- **新增子菜单**

![addmenunode.png](./image/addmenunode.png ':size=720')

## 插件API

- **TreeMenu**

方法名 | 参数| 返回值 | 说明  
------- | ------- | ------- |----- 
`addNodes` | `List<TreeMenuNode>` 一组菜单节点 | `TreeMenu` 树形菜单对象 | 树形菜单增加一组子节点  
`addNodes` | `List<TreeMenuNode>, List<String>` 一组菜单节点，对应的节点图标URL| `TreeMenu` 树形菜单对象 | 树形菜单增加一组子节点 

- **TreeMenuNode**

方法名 | 参数| 返回值 | 说明  
------- | ------- | ------- |----- 
`TreeMenuNode` | `String, String, String` 父节点id，节点id，显示文字 | 无 | 构造方法  
`TreeMenuNode` | `String, String, String, String, String` 父节点id，节点id，显示文字，图标URL，选中图标URL| 无 | 构造方法
`addChild`|`TreeMenuNode`节点对象|`TreeMenuNode`节点对象| 添加单个节点
`addChild`|`int, TreeMenuNode`指定位置，节点对象|`TreeMenuNode`节点对象| 指定节点添加单个子节点
`addChildren`|`List<TreeMenuNode>`节点对象|`TreeMenuNode`节点对象| 添加多个子节点
`getTreeNode`|`String, int`节点id，遍历深度|`TreeMenuNode`节点对象| 根据节点id递归查找节点
`getNodeLevel`|`String, TreeMenuNode`指定位置，节点对象|`TreeMenuNode`节点对象| 指定节点添加单个子节点
`getChildPath`|`String`节点id|`List<String>`节点路径| 指定节点获取节点路径
`iterate`|`int, Consumer<TreeMenuNode>`遍历深度，回调|无| 遍历所有节点
`deleteChildNode`|`String`节点id|`TreeMenuNode`节点对象| 删除指定节点

## 插件示例
- **构造数据并监听事件**

```java
// 插件实现TreeMenuClickListener接口
public class TestPlugin extends AbstractFormPlugin implements TreeMenuClickListener {
  public void registerListener(EventObject event) {
    TreeMenu tree = this.getView().getControl("控件标识");
    tree.addTreeMenuClickListener(this);
  }
  
  public void afterCreateNewData(EventObject event) {
    TreeMenu tree = this.getView().getControl("控件标识");
    // 构造节点数据
    List<TreeMenuNode> nodes = new ArrayList<>();
    
    // 一级节点
    TreeMenuNode level_1_node = new TreeMenuNode();
    // ...
    
    // 二级节点
    TreeMenuNode level_2_node = new TreeMenuNode();
    // ...
    
    level_1_node.addChild(level_2_node);
    
    nodes.add(level_1_node);
    
    // 将整个节点数据add到树控件
    tree.addNodes(nodes);
  }
  
  // 树节点单击时触发
  public void treeMenuClick(TreeNodeEvent evt) {
  
  }
  
  // 树节点双击时触发
  public void treeMenuDoubleClick(TreeNodeEvent evt) {
  
  }
}
```  