# 条形图
> 用宽度相同的条形的高度或长短来表示数据多少的图形。用于`比较`数据，并且想`一眼看出`各个数据的`大小`。

## 控件对象
`kd.bos.form.chart.BarChart`

## 视觉
![条形图](./image/barchart.PNG)

## 属性说明
业务属性 | 属性ID | 属性类型 | 默认值 | 说明 |  
:-------: |  :-------:  | :------: | :-----: | :------
显示标题| `ShowTitle` | boolean | `false` | 控制条形图左上方的标题是否显示
显示图例| `ShowLegend` | boolean | `false` | 控制条形图的图例是否显示，默认为不显示，开启后显示方向是横向
图例纵向展示| `LegendVertical` | boolean | `false` | 控制图例的显示方向为纵向

!>`图例` 是指图上的不同颜色所代表的内容和指标的说明

![图例视觉图](./image/legend.png)

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例

### 业务属性控制

- **插件中声明控件对象**

```java
Control _barchart = this.getControl("barchartap");
if (_barchart instanceof Barchart) {
  BarChart barchart = (BarChart) _barchart;
}
```
- **设置是否显示标题**

```java
boolean isShowTitle = true;
barchart.setShowTitle(isShowTitle);
```
- **设置是否显示图例**

```java
boolean isShowLegend = true;
barchart.setShowLegend(isShowLegend);
```
- **设置图例纵向展示**

```java
boolean isLegendVertical = true;
barchart.setLegendVertical(isLegendVertical);
```

### 事件监听

```java
@Override
public void click(EventObject evt) {
  super.click(evt);
  String key = ((Control)evt.getSource()).getKey();
  if (key.equals("barchartap")) {
    ChartClickEvent e = (ChartClickEvent)evt;
    this.getView().showMessage("SeriesName:" + e.getSeriesName() + "; Name: " + e.getName() + "; Value: " + e.getValue());
  }
}
```

### 高级用法

- **自定义图例（可参照官方示例往legendData里添加需要的配置，下同）**

```java
// 设置两个图例：支出，收入
Map<String, Object> legendData = new HashMap<>();
String[] data = {"支出", "收入"};
lengendData.put("data", data);
barchart.setLegendPropValue("legend",legendData);

// 设置图例位置为中上
barchart.setLegendAlign(XAlign.center, YAlign.top);
```
- **自定义图表标题**

```java
Map<String, Object> titlePropValue = new HashMap<>();
// 主标题内容：text
titlePropValue.put("text", "条形图");

// 副标题内容：subtext
titlePropValue.put("subtext", "barchart");

// 点击副标题跳转的超链接：sublink
titlePropValue.put("sublink", "https://www.echartsjs.com/examples/zh/index.html#chart-type-bar")
barchart.setTitlePropValue("title", titlePropValue)
```
- **自定义提示框组件，包括提示框浮层和坐标指示器**

```java
// 设置触发提示框的类型为axis：坐标轴触发
barchart.addToolTip("trigger", "axis");

// 设置指示器配置项为阴影，更多配置参考echarts官网再添加相应的参数即可
Map<String, Object> axisPointType = new HashMap<>();
axisPointType.put("type", "shadow");
barchart.addToolTip("axisPointer", axisPointType);

// 设置提示框浮层内容格式器，参数含义可查阅echarts官网，这里不一一列举
// 字符串模板格式
List<Object> toolTipFuncPath = new ArrayList<>();
String formatter = "{b0}: {c0}<br />{b1}: {c1}"
barchart.addToolTip("formatter", formatter);
toolTipFuncPath.add("tooltip");
toolTipFuncPath.add("formatter");
barchart.addFuncPath(toolTipFuncPath);

// 回调函数模式
List<Object> toolTipFuncPath = new ArrayList<>();
String formatter = "function(param){if(param.dataIndex===0){return 'placeholder0';} if(param.dataIndex==1){return 'placeholder1';}if(param.dataIndex==2){return 'placeholder2'}}";
barchart.addToolTip("formatter", formatter);
toolTipFuncPath.add("tooltip");
toolTipFuncPath.add("formatter");
barchart.addFuncPath(toolTipFuncPath);
```

- **配置图表的边距（grid位置）**

```java
// 设置图标左边距（更多配置：https://echarts.apache.org/zh/option.html#grid）
barchart.setMargin(Position.left, '10%');
```

- **配置图表的X轴和Y轴**

```java
// 设置X轴的坐标轴名称为日期，类型为category（类目），Y轴的名称为总量，类型为value
Axis x = barchart.createXAxis("日期", "category");
Axis y = barchart.createYAxis("总量", "value");

// 设置X轴类目数据（如果没有设置，则需要再series中设置）
String[] data = {"周一", "周二", "周三", "周四", "周五", "周六", "周日"};
x.setPropValue("data", data);

// 坐标轴刻度标签文字相关设置，格式化刻度标签显示内容
Map<String, Object> data = new HashMap<>();
String dataFormatter = "function (params) {var level = placeholder0;var date = placeholder1;var job = placeholder2;return '职级：' + level[params[0].dataIndex] + '<br />' + '职级时长：' + date[params[0].dataIndex] + '<br />' + '职位：' + job[params[0].dataIndex];}";
data.put("formatter", dataFormatter);
x.setPropValue("axisLabel", data);

// 坐标轴刻度相关设置，设置是否显示坐标轴刻度
Map<String, Object> axisTick = new HashMap<>();
axisTick.put("show", false);
x.setPropValue("axisTick", axisTick);

// 坐标轴线相关设置，设置坐标轴箭头样式
Map<String, Object> axisLine = new HashMap<>();
String[] symbolArr = {"none", "arrow"};
axisLine.put("symbol", symbolArr);
x.setPropValue("axisLine", axisLine);

// 坐标轴在grid区域中的分隔线相关设置，设置分隔线不显示
Map<String, Object> splitLine = new HashMap<>();
splitLine.put("show", false);
x.setPropValue("splitLine", splitLine);
```
- **系列列表，每个系列通过 type 决定自己的图表类型**

```java
// 创建系列为“辅助”的系列
BarSeries barseries = barchart.createSeries("辅助");

// 设置系列名称，用于tooltip的显示，legend 的图例筛选.
barseries.setName("名称");

// 设置数据堆叠，同个类目轴上系列配置相同的stack值可以堆叠放置
barseries.setStack("总量");

// 设置系列中数据的内容
int[] number = {1, 2, 3, 4, 5, 6}
barseries.setData(number);

// 设置柱条颜色（直接设置颜色值和渐变设置）
barseries.setColor('rgba(0,0,0,0)');
barseries.setColor("new echarts.graphic.LinearGradient(0, 0, 1, 0, [{offset: 0, color: '#B177FF' },{offset: 1, color: '#828FFF'}], false)");
```

- **传递新的图表数据时，不与原来图表数据合并渲染**
```java
// 设置Option中的isMerge属性，默认是true
barchart.setMerge(false);
```

- **option自定义，更灵活地配置图表**
```java
// 以下逻辑需要在afterBindData里调用才会生效
IClientViewProxy clientViewProxy = (IClientViewProxy)this.getView().getService(IClientViewProxy.class);
// 其中，barchartap是图表在设计器里的标识（可根据设计器设置然后改变），data是属性名（写死不可变），stringObjectMap是整个图表所需要的Option（是一个Map）
// 假如，图表中有用到formatter函数，例如tooltip使用formatter自定义，你需要的是在Option里添加一个functions属性：Option = { functions: [] }
// 如果你使用过addFunctionPath这个接口，你就会发现它会往Option里添加这个functions属性，你试着按照格式拷贝一份过去就可以了
// 提供一份Option的简单示例：https://echarts.apache.org/examples/zh/editor.html?c=bar-background
clientViewProxy.setFieldProperty("barchartap", "data", stringObjectMap);
```

!> 注意事项:  
1.`tooltip、X轴和Y轴、series`中，只要`formatter`使用到了`回调函数`的格式，均需要通过`barchart.addFunctionPath(formatter)`才会生效；
2.其他未提及的参数的设置，均可参考echarts官网得知参数（https://echarts.apache.org/zh/option.html#title），使用setPropValue接口设置。

?>柱状图、饼图、点线图、雷达图、仪表盘的配置和相关API和条形图无太大差异
