# 锚点控件
>点击锚点上不同链接，可以在单据(或跨单据)页面上不同显示位置(容器/其他控件)间跳转(滚动)。
>即书签效果。

## 控件对象
`kd.bos.ext.form.control.AnchorControl`

## 视觉展示

- **默认(纵向模式锚点)**

![default.png](./image/anchor_control_vertical.png) 

- **横向模式锚点** 

![playing.png](./image/anchor_control_horizon.png) 


## 控件属性说明

属性名 | 属性ID | 类型 | 说明 
---- | :----: | :----: | ---- 
布局方向 | `direction` | int | 默认纵向，可配置为`横向模式`展示
自动高亮 | `hl` | bool | 默认false，属性打开后，点击锚控件上链接，目标滚动到可见区域后自动高亮提醒位置
允许收起 | `foldable` | bool | 默认false，属性打开后，锚点面板默认隐藏，只显示一个”展开“的小图标，鼠标悬停至”展开“图标，锚点面板展开；鼠标移出面板，面板消失。

控件通用属性请参考=>[传送门](advanced/controlProperties/)

## 功能说明

功能名 | 说明
------- | ------- 
页面内容区跳转 | 根据设定的锚点(链接)数据，点击不同链接，自动跳转(滚动)到链接对应的特定区域/容器/其他控件去，即将目标滚动到当前可见区域来。
显示模式 | 默认纵向显示，可配置为横向显示。横向显示只取第一层级链接数据，子层级不处理。
自动高亮 | 点击链接跳转，目标对象会自动高亮，提示用户是具体是哪个区域或控件等。

## 其他说明

- 该控件支持的相关属性在设计器中`暂未开放`，目前仅支持从插件中设置，具体请看`插件示例`章节
- 控件数据源的设置在服务端没有封装为单独对象，所以需要自己在插件中去拼装

## 锚点数据结构示例
```javascript
{
      target: '控件id3',
      caption: '个人信息',
      items: [ //子层级数据
        {
          target: '控件id4',
          caption: '教育信息',
          items: []
        },
        {
          target: '控件id5',
          caption: '项目经历',
          items: []
        },
        {
          target: '控件id6',
          caption: '社会关系',
          items: []
        }
      ]
    }
```
## 插件示例

- **插件代码**

```java
public void do() {
  // 设置显示模式(横向/纵向)
  Map<String, Object> dataMap = new HashMap<>();
  dataMap.put("direction", 1); //设置为横向
  dataMap.put("k", "控件id");
  IClientViewProxy proxy = this.getView().getService(IClientViewProxy.class);
  proxy.addAction(ClientActions.updateControlStates, dataMap);

  //设置滚动自动高亮
  AnchorControl anchorCtl = this.getView().getControl("控件id");
  anchorCtl.setHighlight (true);

  // 设置数据源
  AnchorControl anchorCtl = this.getView().getControl("控件id");
  anchorCtl.addItems(List<AnchorItems> list); //List<AnchorItems>: 锚点数据链接信息
}
```

