# 附件面板
>用于上传文件的控件，支持文件选择、拖拽上传文件，同时支持文件下载预览等。

## 控件对象
`kd.bos.form.control.AttachmentPanel`

## 视觉展示

- **非折叠状态**
![attachment_collapse.png](./image/attachment_collapse.png)

- **折叠状态**
![attachment.png](./image/attachment.png)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ----
可折叠 | `collapsible` | boolean | `false` | 设置附件面板是否折叠
智能折叠 | `defaultcollapse` | boolean | `false` | 表单初始加载时根据有无附件智能折叠，`可折叠`属性打开后才可见该属性
扩展名 | `ext` | string | | 设置可上传文件扩展名，例：`zip,rar`  
显示创建人 | `sct` | boolean | `true` | 设置已上传文件条目是否显示创建人
显示创建日期 | `scd` | boolean | `true` | 设置已上传文件条目是否显示上传日期
显示备注 | `smk`| boolean | `true` | 设置已上传文件条目显示`备注`按钮，可`点击`配置附件备注
显示附件数 | `san` | boolean | `false` | 显示已经上传的文件数量，在可折叠状态下与非折叠态下有所区别
最大附件大小 | `maxatmsize` | number | `20M` | 设置单个可上传文件大小，在系统参数中可设置服务器最大附件的大小，优先级比设计器属性要高
最大附件数 | `maxatmcount` | number | | 设置附件可上传的文件数量，超出限制则不可上传
图片显示缩略图 | `istn` | boolean | `false` | 上传的图片可以`缩略图`的形式展现，其他格式不变
显示附件操作按钮 | `attachsob` | boolean | `false` | 显示附件操作按钮，不设置默认为hover显示(目前只支持插件配置)
单据实体 | | | | 在两个相同实体单据之间共享附件
日期格式 | `attachdf` | string | `date` | **date**(短日期: 年月日, 如 `2020-05-28`); **datetime**(长日期, 年月日 时分秒, 如 `2020-05-28 10:00:00`)

?> 控件通用属性[请参考](advanced/controlProperties/)

## 功能说明  :id=attachgn
功能名 | 说明
:----: | ----
上传 | 上传方式包括点击按钮上传、点击上传区域上传、拖拽文件上传、粘贴截图上传（IE浏览器不支持）
删除 | 删除功能可控制权限；可记录单个文件下载次数
下载 | 可单个下载，也可全部下载，单据保存后，将所有文件压缩为`zip`包下载
预览 | 保存的附件可预览；目前附件可预览的文件类型包括`xlsx、docx、pptx、txt、pdf、png、jpg、gif`等；预览功能需要通过修改系统参数打开，指令为`fileserver.attachment.preview = true`；可记录单个文件预览次数
自定义操作按钮 | 可通过插件给附件增加除`删除、下载、预览`操作以外的操作按钮，满足多样化需求

## 插件示例
- **插件监听附件上传、删除、添加备注事件**

```java
// 第一步：插件实现 UploadListener、AttachmentMarkListener 接口
public class TestPlugin extends AbstractFormPlugin implements UploadListener, AttachmentMarkListener {
  @Override
  public void registerListener(EventObject e) {
    // 第二步：附件面板添加监听
    AttachmentPanel panel = this.getView().getControl("控件标识");
    panel.addMarkListener(this);
    panel.addUploadListener(this);
  }

  @Override
  public void upload(UploadEvent evt) {
    // 文件上传前进入该方法
  }
  
  @Override
  public void afterUpload(UploadEvent evt) {
    // 文件上传后进入该方法
  }

  @Override
  public void remove(UploadEvent evt) {
    // 删除附件进入该方法
  }

  @Override
  public void mark(AttachmentMarkEvent evt) {
    // 提交备注前进入该方法，可修改备注信息
  }
}
```  

- **插件控制附件删除权限、记录删除预览次数事件** 

```java
// 第一步：插件实现 BeforeAttachmentRemoveListener, AttachmentPreviewListener, AttachmentDownloadListener 接口
public class TestPluginForm extends AbstractFormPlugin implements BeforeAttachmentRemoveListener, AttachmentPreviewListener, AttachmentDownloadListener {
	
	private static final String ATTACHMNETEDIT = "attachmentedit";
	private static final String ATTACHMNETPANEL = "attachmentpanel";
	
	@Override
	public void registerListener(EventObject e) {
		//附件字段注册监听事件
		AttachmentEdit edit = this.getControl(ATTACHMNETEDIT);
		edit.addBeforeRemoveListener(this);
		edit.addPreviewListener(this);
		edit.addDownloadListener(this);
		
		//附件面板注册监听事件
		AttachmentPanel panel = this.getControl(ATTACHMNETPANEL);
		panel.addBeforeRemoveListener(this);
		panel.addPreviewListener(this);
		panel.addDownloadListener(this);
	}

	@Override
	public void beforeAttachmentRemove(BeforeAttachmentRemoveEvent evt) {
		Control control = (Control) evt.getSource();
		if (ATTACHMNETEDIT.equals(control.getKey())) {
			//取消删除
			evt.setCancel(true);
		}else if (ATTACHMNETPANEL.equals(control.getKey())) {
			
		}
	}
	
	@Override
	public void attachmentAfterPreview(AttachmentPreviewEvent evt) {
	}
	
	@Override
	public void attachmentAfterDownload(AttachmentDownLoadEvent evt) {
	}
}
```  

- **插件控制附件新增自定义按钮** 

```java
public class TestPluginAttachment extends AbstractBillPlugIn implements AttachmentOperaClickListener {

	private static final String KEY_BUTTON = "buttonap";
	private static final String KEY_ATTACHMENTPENAL = "attachmentpanel";
	
	@Override
	public void registerListener(EventObject e) {
		this.addClickListeners(KEY_BUTTON);
		AttachmentPanel panel = this.getControl(KEY_ATTACHMENTPENAL);
		panel.addOperaClickListener(this);;
	}
	
	@Override
	public void click(EventObject evt) {
		Control ctl = (Control) evt.getSource();
		if (KEY_BUTTON.equals(ctl.getKey())) {
			AttachmentPanel panel = this.getControl(KEY_ATTACHMENTPENAL);
			List<AttachBtnOption> btns = new ArrayList<AttachBtnOption>();
			btns.add(new AttachBtnOption("attoption_1", new LocaleString("分享")));
			btns.add(new AttachBtnOption("attoption_2", new LocaleString("打印")));
			panel.addAttachOperaBtn(btns );
		}
	}
	
	@Override
	public void attachmentOperaClick(AttachmentOperaClickEvent e) {
		AttachmentPanel panel = (AttachmentPanel) e.getSource();
		Map<String, String> attinfo = (Map<String, String>) e.getAttachmentInfo();
		List<Map<String, Object>> atts = panel.getAttachmentData();
		Map<String, Object> att = null;
		for (Map<String, Object> map : atts) {
			String uid = (String) map.get("uid");
			if (uid.equals(attinfo.get("uid"))) {
				att = map;
				continue;
			}
		}
		if ("attoption_1".contentEquals(e.getOperaKey())) {
			System.out.println("分享附件："+att.get("name")+att.get("url"));
		}
	}
}
```  

- **通过U指令修改创建日期格式** 

```java
public class TestPluginUpdateCteateDateType extends AbstractBillPlugIn implements AttachmentOperaClickListener {

	private static final String KEY_ATTACHMENTPENAL = "attachmentpanel";

	@Override
	public void updateDateFormatter() {
		IClientViewProxy clientViewProxy = this.getView().getService(IClientViewProxy.class);
		Map<String, Object> dataMap = new HashedMap();
		dataMap.put("k", KEY_ATTACHMENTPANEL);
		dataMap.put("attachdf", "datetime");
		clientViewProxy.addAction(ClientActions.updateControlStates, dataMap);
	}
}
```  
