# 自定义控件
>当平台提供的控件不能完全满足业务所需要的场景时，我们可以使用自定义控件重新去实现一个全新的控件。自定义控件从前端到后端完全你自主开发，所以更适合有前端开发基础开发者。

?>如果你从没开发过苍穹自定义控件，建议你先从自定义控件开发入门开始[传送门](./advanced/customcontrol/)，跟着我们一步步完成一个简单的例子开发后，再来看该文档收获更大哦！

## 控件对象

- **设计时对象**
`kd.bos.ext.metadata.form.control.CustomControlAp`

- **运行时对象**
`kd.bos.ext.form.control.CustomControl`

!>动态创建自定义控件时，`配置项`和`控件方案`只能在设计时配置，运行时自定义控件可以在插件里调用setData方法给自定义控件传输业务数据。

## 视觉
自定义控件没有设计时，开发者只需关注运行时逻辑即可，以下图片显示的是使用自定义控件完成的例子效果图：
![demo1](./image/demo1.png ':size=420')
![demo2](./image/demo2.png ':size=300')
![demo3](./image/demo3.png)

## 属性说明
### 控件方案（属性ID：schemaInfo）：Map类型
每一个自定义控件对应一个控件方案，选择或者新建一个控件方案，自定义控件会根据关联的方案的渲染对应的页面

属性名 | 属性ID | 属性类型 | 说明
:-------: | :-------:  | :------: | :------
控件方案id | `schemaId`  | String | `必填项`，不同控件的方案id不同，schemaId是控件`唯一性`的标识
开发商标识 | `isvId`  | String | `必填项`，例如：kingdee，kddep等
领域标识 | `moduleId`  | String | `必填项`，例如：hr，fi等
控件方案名称 | `schemaName`  | String | `非必填项`，设置描述控件的名称

### 配置项（属性ID：ci）：List类型
用于在设计时给自定义控件配置自定义参数，可用于自定义业务属性和样式属性，自定义控件内部逻辑可根据不同的属性值来处理不同的渲染逻辑。

!> 建议配置项用来配置控件的元数据，`不推荐通过此属性传输业务数据给控件`

## 模型（model）
模型对象，自定义控件默认提供，值不能修改，model中的内容通常是包含该自定义控件的外部属性，其中包含以下内容

属性名 | 属性ID | 类型 | 说明
------ | :------ | :------ | :------
页面标识 | `pageId` | String | 自定义控件所在表单的data-page-id
控件标识 | `key` | String | 设计时配置的标识，是唯一的
DOM元素 | `dom` | Node | 自定义控件提供的可供设置innerHTML的DOM节点
控件方案id | `scheamId` | String | 
开发商标识 | `isvId` | String |
领域标识 | `moduleId` | String |
指令函数 | `invoke` | Function | 将自定义控件事件发送给平台后端，接收两个参数，eventName 执行事件名称，eventArgs 执行事件所需参数，推荐使用字符串

## 数据（props）
数据对象，props里面描述的通常是该控件的内部属性，直接和控件渲染逻辑相关，常用如下：

属性名 | 属性ID | 类型 | 说明
------ | :------ | :------ | :------
主题色 | `themeColor` | String | 可以根据该主题色标示实现自定义控件多主题
锁定性 | `lock` | boolean | 可以在设计时的业务属性上配置锁定性，根据该属性控件控件是否可编辑
数据对象 | `data` | any | 插件中setData接口设置的数据
配置项 | `configItems` | List | 控件设计时自定义属性
卡片行数据 | `cardRowData` | Object | 当自定义控件放在卡片列表、卡片分录中时，通过cardRowData就能获取到卡片的相关信息，包括卡片里的字段、控件，还有卡片行号


## 后端插件控件方法
接口名称 | 参数 | 说明  
------- | ------- | -------  
`getData` | 无 | 获取自定义控件数据，目前只有标识
`setData` | Object | 通过该方法来设置自定义控件需要的业务数据
`setConfigItems` | List<Map<String, String>> | 设置配置项数据


## 前端JS API
在`window`下面封装了一个对象`KDApi`，通过该对象可获取平台提供的自定义控件API，常用方法如下

方法名称 | 说明  
------- | ------- 
`KDApi.register` | 将控件在运行时注册到平台中，该方法每自定义控件必须调用
`KDApi.loadFile` | 加载`JS`、`CSS`，需加载index.js以外的文件时使用
`KDApi.getHTMLStringBytemplate` | 通过HTML模板和数据，生成HTML字符串，可以赋值给innerHTML
`KDApi.getTemplateStringByFilePath` | 根据字符串模板的文件路径和数据，获取HTML字符串，可以赋值给innerHTML 
`KDApi.getNameSpace` | 获取控件的命名空间，命名控件由开发商标识、领域标识、控件方案id组成
`KDApi.getLangMsg` | 根据templateInfo，在langObj中找出词条

### KDApi.register

将控件注册到平台中，该方法每自定义控件必须调用。

**参数**
- **`type(String)`** 自定义控件唯一标识，必须与schemaId的值一致
- **`ctrl(Object)`** 自定义控件对象，使用函数式声明，在运行时会生成实例

**返回**
无返回值

**例子**
```javascript
// 创建控件对象
function MyComponent (model) {
  // ...
}

// 将控件注册到平台中
window.KDApi.register('schemaId的值', MyComponent)
```

### KDApi.loadFile

动态加载`JS`、`CSS`文件

**参数**
- **`filePath(String|Array)`** 文件路径或者文件路径数组，支持相对路径
- **`model(Object)`** 控件`model`对象
- **`callback(Function)`** 回调方法

**返回**
无返回值

**例子**
```javascript
// 加载单个JS文件
window.KDApi.loadFile('./js/a.js', model, callback).then(function () {
  // do something
  callback()
})

// 如多个文件没有先后依赖关系的，可以并行加载多个文件，如
window.KDApi.loadFile(['./js/a.js', './js/b.js', './css/c.css'], model, callback).then(function () {
  // do something
  callback()
})
```

### KDApi.getHTMLStringBytemplate

 通过HTML模板和数据，生成HTML字符串，可以赋值给innerHTML

**参数**
- **`tpl(String)`** HTML模板
- **`data(Object)`** 生成HTML字符串所需要的数据对象

**返回**
HTML字符串

**例子**
```javascript
const tpl = '<div>{{name}}</div>'
const data = {name: '张三'}
const result = window.KDApi.getHTMLStringBytemplate(tpl, data)
// => '<div>张三</div>'
```

### KDApi.getTemplateStringByFilePath

根据字符串模板的文件路径和数据，获取HTML字符串，可以赋值给innerHTML

**参数**
- **`filePath(String)`** HTML模板文件路径，支持相对路径
- **`model(Object)`** 控件`model`对象，
- **`data(Object)`** 数据对象

**返回**
HTML字符串

**例子**
```javascript
window.KDApi.getTemplateStringByFilePath('./html/tpl.html', model, 
{name: '张三'}).then(function (result) {
  // => '<div>张三</div>'
  const dom = document.getElement('xxx')
  // model.dom.innerHTML = result
  dom.innerHTML = result
})
```

### KDApi.getNameSpace

获取控件的命名空间，命名控件由开发商标识、领域标识、控件方案id组成

**参数**
- **`model(Object)`** 控件`model`对象

**返回**
无返回值

**例子**
```javascript
// 获取hr的自定义label控件的命名空间
const nameSpace = window.KDApi.getNameSpace(model)
// => './isv/kingdee/hr/label'
```

### window.KDApi.getLangMsg

根据templateInfo，在langObj中找出词条

**参数**
- **`model(Object)`** 自定义控件model 
- **`key(String)`** 词条文件中的key
- **`templateInfo(Object)`** {key: value}，替换词条模板中的变量，key为变量名

**返回**
词条

**例子**
```javascript
// 在zh_CN.json中
{
  "dellabel.key0001": "{text}" // 其中text是variables中的text值
}

// 接口使用
var str = KDApi.getLangMsg('dellabel.key0001', { text: "标签" })
```

## 生命周期

### `_setModel`
自定义控件对象实例化时绑定model的方法
### `init`
组件三个生命周期方法之一，组件初始化进入的方法，可以将渲染组件或者DOM操作相关的函数放在此处
### `update`
组件数据更新时触发该方法，可以将数据更新后的需要进行相关操作的函数放在此处
### `destoryed`
组件卸载时触发该方法，可以在这里清除绑定在DOM上的事件和进行缓存清除工作

**例子**
```javascript
function MyComponent (model) {
    this._setModel(model)
}

MyComponent.prototype = {
  _setModel: function (model) {
    this.model = model
  },
  init: function (props) {
    // doSomething()
    console.log('-----init', this.model, props)
  },
  update: function (props) {
    // doSomething()
    console.log('-----update', this.model, props)
  },
  destoryed: function () {
    // To Do Clear
    console.log('-----destoryed', this.model)
  }
}
```


## 前端示例

- **实现一个控件方案id为dellabel的自定义控件**

```javascript
// index.js

(function (KDApi, $) {
  function MyComponent(model) {
      this._setModel(model);
  }

  // 主题色
  var themeColor;

  MyComponent.prototype = {
      _setModel: function (model) {
          this.model = model;
      },
      init: function (props) {
          themeColor = props.themeColor
          setHtml(this.model, props);
      },
      update: function (props) {
          themeColor = props.themeColor;
          updateHtml(this.model, props);
      },
      destoryed: function () {

      }
  }

  var setHtml = function (model, props, isUpdate) {
      var cssHref = KDApi.getNameSpace(model) + './css/dellabel.css';
      if($('link[href="' + cssHref + '"]').length === 0){
          KDApi.loadFile('./css/dellabel.css', model, function() {
              updateHtml(model, props);
          })            
      }
      updateHtml(model, props);
  }

  var updateHtml = function(model, props){
      var template = '<div class="hr_delLabel" title="<%= text %>">' +
                          '<div class="hr_delLabel_text">' +
                              '<%= text %>' +
                          '</div>' +
                          '<div class="hr_delLabel_icon"></div>' +
                      '</div>';
      var result = KDApi.getHTMLStringBytemplate(template, {
          text: props.data && props.data['text'] ? props.data['text'] : props.configItems ? props.configItems[0].value : '标签'
      })
      model.dom.innerHTML = result;
      initEvent(model, props);
  }

  var initEvent = function (model, props) {
      $(model.dom).css({
          "overflow":"visible"
      })
      $('.hr_delLabel', model.dom).css({
          "width": model.dom.style.width,
          "height": model.dom.style.height,
          "font-size": model.dom.style.fontSize !== '' ? model.dom.style.fontSize : '12px',
          "text-align": model.dom.style.textAlign,
          "overflow":"visible"
      }).click(function () {
          model.invoke('click', '');
          model.invokeCustomMethod('click', [0])
      }).mouseenter(function () {
          $(this).find('.hr_delLabel_icon').css({
              "background-color": themeColor,
          }).addClass('active');
          $(this).css({
              "border-color": themeColor
          })
      }).mouseleave(function () {
          $(this).find('.hr_delLabel_icon').css({
              "background-color": "transparent"
          }).removeClass('active')
          $(this).css({
              "border-color": "#CCCCCC"
          })
      })
      $('.hr_delLabel_text', model.dom).css({
          "width": parseInt(model.dom.style.width) - 19 + 'px',
          "line-height": model.dom.style.height
      })
  }
  KDApi.register('dellabel', MyComponent)
})(window.KDApi, jQuery);
```

```css
/* delLabel.css  */

.hr_delLabel {
  display: flex;
  box-sizing: border-box;
  flex-direction: row;
  align-items: stretch;
  font-size: 0px;
  border: 1px solid #CCCCCC;
  border-radius: 2px;
}

.hr_delLabel_text {
  padding: 0 10px;
  background-color: transparent;
  text-align: inherit;
  font-size: inherit;
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}

.hr_delLabel_icon {
  position: relative;
  width: 20px;
  background-color: transparent;
  flex-grow: 0;
  cursor: pointer;
}

.hr_delLabel_icon::before,
.hr_delLabel_icon::after {
  content: "";
  position: absolute;
  top: 50%;
  right: 2px;
  height: 1px;
  width: 14px;
  background: #838383;
}
.hr_delLabel_icon::before {
  transform: rotate(45deg);
}
.hr_delLabel_icon::after {
  transform: rotate(-45deg);
}
.hr_delLabel_icon.active::before,
.hr_delLabel_icon.active::after{
  background: #ffffff;
}

```

- **打包成.zip格式，根目录必须有index.js**

![package.PNG](./image/package.PNG)

?> zip包的命名对于程序的运行没有任何影响，推荐命名规范：领域 _ 应用 _ schemaId。[点击此处下载](./ctrl/commonControl/customctrl/example/dellabel.zip ':ignore')完整例子。

## 插件示例
- **插件监听发出的事件**

```java
public class TestPlugin extends AbstractFormPlugin {

  @Override
  public void customEvent(CustomEventArgs e) {
    // 自定义控件通过model.invoke(eventName, data) 方法发出的请求都会进入该方法
    // 插件重载这个方法即可
    String key = e.getKey();
    switch (key) {
      case "":
      
      break;
      
      default:
    
    }
  }
}
```

- **插件给自定义控件传输数据**

```java
@Override
public void afterBindData(EventObject e) {
  super.afterBindData(e);
  // 获取控件
  CustomControl customcontrol = this.getView().getControl('标识');
  Map<String, String> data = new HashMap<>();
  data.put("text", "这是一段文本");
  // 通过setData给自定义控件传输数据，前端就能通过props.data获取数据
  customcontrol.setData(data);
}
```

- **动态创建自定义控件**

```java
// 引入设计时的jar包
import kd.bos.ext.metadata.form.control.CustomControlAp;
import kd.bos.entity.CtlSchemaInfo;

// 声明设计时自定义控件的对象
private CustomControlAp customControlAp = new CustomControlAp();

/**
  * 设置控件方案,平台控件方案该了传输格式，我们这边需要相对应的进行调整
  * {"schemaId":"dellabel","isvId":"开发商标识","moduleId":"领域标识"}
  * 目前来看，这三个值我们只需要使用方案ID,所以只需要在基类这里增加Map类型的数据就可以，不影响下层使用
  * @param ctlSchema
*/
public void setCtlSchema(String ctlSchema){
  Map<String, String> schemaMap = new HashMap<>();
  schemaMap.put("schemaId", ctlSchema.toLowerCase());
  schemaMap.put("isvId", "此处写开发商标识");
  schemaMap.put("moduleId", "此处写领域标识");
  CtlSchemaInfo ctlSchemaInfo = new CtlSchemaInfo(ctlSchema.toLowerCase(), ctlSchema.toLowerCase(), "此处写开发商标识", "此处写领域标识");
  customControlAp.setCtlSchemaInfo(ctlSchemaInfo);
}
/**
  * 设置配置项
  * @param key
  * @param value
*/
public void setConfigItem(String key, String value){
  Map<String, String> configItem = new HashMap<>();
  configItem.put(key, value);
  configItems.add(configItem);
  customControlAp.setConfigItems(configItems);
}
// 其他基本的业务和样式属性的设计时设置同其他控件一致

// 设置控件方案和配置项
setCtlSchema("dellabel");
setConfigItem("text", "这是一段文本");
```

- **通过showForm指令将表单show到自定义控件的DOM元素里**

```java
@Override
public void afterBindData(EventObject e) {
  super.afterBindData(e);
  FormShowParameter parameter = new FormShowParameter();
  parameter.setFormId("表单的formId");
  parameter.getOpenStyle().setShowType(ShowType.InContainer);
  parameter.getOpenStyle().setTargetKey("自定义控件的标识");
  this.getView().showForm(parameter);
}
```

## 高级进阶

自定义控件默认是集成了jquery，简单控件可直接使用其进行开发即可，如果你希望通过使用其他框架来实现如React、Vue等，也是支持的，以下将以热门框架React、Vue为例，在例子中使用前端打包工具webpack，开发完后，编译，将控件目录下的lang文件夹、css文件夹、index.js、index.js.map打成zip包上传即可。

### Vue

- **在此例子中打包入口为main.js，输出index.js和index.css，所以之前在index.js里写的代码可以挪到main.js中**

```javascript
// 在main.js上通过import引入Vue库和自己写的Vue组件库
import Vue from 'vue'
import Steps from './components/steps.vue'
import eventBus from '../../../../../../util/eventBus'

/**
 * Vue实例在setHtml方法中声明，初始化执行init的时候就能够创建
 * 声明Vue实例对象时，在其挂载完毕的生命周期里声明一个订阅，用于订阅update方法中发布的消息，从而更新实例数据
 * update方法中，发布一个消息，让Vue实例接收消息，更新数据
 * 在Vue实例的destroyed中，结束订阅
 * 注意loadFile中index.css的引入路径，因为webpack打包后将其放在了css文件夹里，所以路径是./css/index.css
 */
(function (KDApi) {
  function MyComponent (model) {
    this._setModel(model)
  }

  MyComponent.prototype = {
    _setModel: function (model) {
      this.model = model
    },
    init: function (props) {
      console.log('-----init', this.model.style, props)
      setHtml(this.model, props)
    },
    update: function (props) {
      console.log('-----update', this.model, props)
      eventBus.pub(this.model, 'update', props)
    },
    destoryed: function () {
      console.log('-----destoryed', this.model)
    }
  }

  const setHtml = (model, props) => {
    KDApi.loadFile('./css/index.css', model, () => {
      const { invoke, dom } = model
      const { data } = props
      const activeI = data ? data.avtiveIndex : -1
      new Vue({
        el: dom,
        template: '<Steps :invoke="invoke" :activeInd="activeIndex" :model="model" :getLangMsg="getLangMsg" />',
        components: {
          Steps
        },
        data: {
          activeIndex: activeI,
          model: model
        },
        mounted () {
          const self = this
          this.updateSub = eventBus.sub(model, 'update', (props) => {
            const { data } = props
            self.activeIndex = data ? data.activeIndex : -1
          })
        },
        destroyed () {
          eventBus.unsub(this.updateSub)
        },
        methods: {
          invoke: (eventName, args) => {
            invoke(eventName, args)
          },
          getLangMsg: (key) => {
            return KDApi.getLangMsg(model, key)
          }
        }
      })
    })
  }

  // 注册自定义组件
  KDApi.register('stepsVue', MyComponent, {
    isMulLang: true
  })
})(window.KDApi)

```

### React

- **在此例子中打包入口为main.js，输出index.js和index.css，所以之前在index.js里写的代码可以挪到main.js中**

```javascript
import React from 'react'
import ReactDOM from 'react-dom'
import TodoList from './components/TodoList'
import eventBus from '../../../../../../util/eventBus'

/**
 * 在setHtml中声明Root类，使用ReactDOM.render将其渲染在model.dom中
 * 在Root类的componentDidMount里，声明一个订阅，用于接收后端更新发过来的消息，从而去更新组件
 * 在update里，发布一个消息，当后端插件给自定义控件传递新数据时，就能将消息发布给Root
 * 在Root类的componentWillUnmount里，取消订阅
 * 在destoryed里，使用ReactDOM.unmountComponentAtNode卸载Root
 * 注意loadFile中index.css的引入路径，因为webpack打包后将其放在了css文件夹里，所以路径是./css/index.css
 */
(function (KDApi) {
  function MyComponent (model) {
    this._setModel(model)
  }

  MyComponent.prototype = {
    _setModel: function (model) {
      this.model = model
    },
    init: function (props) {
      console.log('-----init', this.model.style, props)
      setHtml(this.model, props)
    },
    update: function (props) {
      console.log('-----update', this.model, props)
      eventBus.pub(this.model, 'update', props)
    },
    destoryed: function () {
      console.log('-----destoryed', this.model)
      ReactDOM.unmountComponentAtNode(this.model.dom)
    }
  }

  var setHtml = function (model, primaryProps) {
    KDApi.loadFile('./css/index.css', model, () => {
      class Root extends React.Component {
        constructor(props) {
          super(props)
          this.state = {
            customProps: props.customProps,
            model: props.model
          }
        }
        componentDidMount () {
          const { model } = this.state
          this.updateSub = eventBus.sub(model, 'update', (updateProps) => {
            this.setState({ customProps: updateProps})
          })
        }
        shouldComponentUpdate () {

        }
        componentWillUnmount () {
          eventBus.unsub(this.updateSub)
        }
        render () {
          const { customProps, model } = this.state
          return (
            <TodoList model={model} customProps={customProps} />
          )
        }
      }
      ReactDOM.render(<Root model={model} customProps={primaryProps} />, model.dom)
    })
  }

  // 注册自定义组件
  KDApi.register('todolistreact', MyComponent)
})(window.KDApi)

```

!> 引用第三方库，出现了版本冲突需要自己处理，平台只提供jQuery。

?> [点击此处下载](./ctrl/commonControl/customctrl/example/customcontrol-example.zip ':ignore')以上三个模板的案例。

### 本地调试

在请求头的query部分末尾加上`kdcus_cdn=http://localhost:3001`，如果有hash，则在hash末尾也加上，就可以加载你本地写的自定义控件的代码，开发过程中无需打包上传，也无需修改代码后重启整个服务，方便快捷。

!> 此功能在补丁BOS_V2.0.024才有。

例如：  
<u>https://feature.kingdee.com:2024/baseline_a/?formId=pc_main_console#/dform?formId=pc_main_console</u>  
改为：  
<u>https://feature.kingdee.com:2024/baseline_a/?formId=pc_main_console&kdcus_cdn=http://localhost:3001#/dform?formId=pc_main_console&kdcus_cdn=http://localhost:3001</u>

?> `http://localhost:3001`这里的url由你本地启动的服务决定，这里只是举例。

- **使用express起一个静态资源服务**

```javascript
// server.js
const express = require("express"); // npm i --save express
const app = express();
const path = require('path')
const serveIndex = require('serve-index');  // npm i --save serve-index

//设置跨域访问
app.all('*', function (req, res, next) {
    // 设置可以跨域访问的域名，比如https://feature.kingdee.com:2024
    res.header('Access-Control-Allow-Origin', 'https://feature.kingdee.com:2024');
    res.header('Access-Control-Allow-Credentials', true);
    res.header('Access-Control-Allow-Headers', 'Content-Type,Content-Length, Authorization, Accept,X-Requested-With')
    res.header('Access-Control-Allow-Methods', 'PUT,POST,GET,DELETE,OPTIONS');
    res.header('Content-Type', 'text/css;text/javascript;text/html;application/json;application/x-img');
    next()
});

// 以下两行代码设置public文件夹为静态资源文件夹，访问http://localhost:3001就可以访问到public文件夹
app.use('/', express.static(path.join(__dirname, './src/web')));
app.use('/', serveIndex(path.join(__dirname, './src/web'), {'icons': true}));

// 设置监听3001端口，可以依据具体情况自行修改
app.listen(3001, () => { // 默认为localhost（本地主机）; 端口号3001
    console.log("CustomControl Static Server is Running!")
})

```

- **通过nodemon启动server.js，服务文件和静态资源文件一修改，就会自动重启服务**
```javascript
npm install -D nodemon
nodemon server.js
```

- **将自定义控件的目录放在`src/web/isv/开发商标识/领域标识/`下**

!> 可以根据开发商标识、领域标识自行建立对应命名的文件夹

?> [点击此处下载](./ctrl/commonControl/customctrl/example/customcontroldev.zip ':ignore')以上示例

## 多语言

自定义控件的`index.js`中所用到的中文，可以抽成中文词条，写在`lang/zh_CN.json`里。`lang`文件夹建立在控件包根目录下（与index.js同级），同时需要什么语言就建对应的文件，比如英文：en_US.json，自己翻译。

- **在控件下建一个lang目录**
![createLangDir](./image/createLangDir.png)

- **在lang文件夹下新建zh_CN.json文件**
![lang](./image/lang.png)

### 代码示例

将index.js的中文词条抽出，放在zh_CN.json中，以下用自定义控件dellabel为例：

- **代码中有中文词条”标签“，则在zh_CN.json文件中这样子写：**
```javascript
{
    "dellabel.key0001": "标签1",   // 假如这是第一个词条
    "dellabel.key0002": "标签2"   // 假如这里有第二个词条
    "dellabel.key0003": "标签3{date}" // 假如这里有第三个词条，且词条里面含有一个变量date
    "dellabel.key0004": "标签4{date}, {text}" // 假如这里有第四个词条，且词条里面含有两个变量：date和text
}
```
- **在代码里，词条的位置在这：**

![citiao](./image/citiao.png)

- **则需要这样子改：**
```javascript
var updateHtml = function(model, props){
  var lang = model.lang  // 获取自定义控件所在单据语言环境，默认中文
  var template = '<div class="hr_delLabel" title="<%= text %>">' +
                      '<div class="hr_delLabel_text">' +
                          '<%= text %>' +
                      '</div>' +
                      '<div class="hr_delLabel_icon"></div>' +
                  '</div>';
  var result = KDApi.getHTMLStringBytemplate(template, {
      text: props.data && props.data['text'] ? props.data['text'] : props.configItems ? props.configItems[0].value : KDApi.getLangMsg(model, 'dellabel.key0001')
      /*
        * KDApi.getLangMsg(model, 'dellabel.key0003', { date: new Date() })
        * KDApi.getLangMsg(model, 'dellabel.key0004', { date: new Date(), text: '文字内容' })
      */
  })
  model.dom.innerHTML = result;
  initEvent(model, props);
}
```

- **在KDApi.register中，需要这样子改：**
```javascript
/*
  * KDApi.register新增第三个参数：config，object类型，没有传入时默认为空
  * isMulLang：是否使用多语言， 默认为false
*/
KDApi.register('dellabel', MyComponent, {
  isMulLang: true
})
```
