# IFrame
>用来显示第三方链接进来的网站内容，当与第三方合作开发时，如有需要引入对应模块，可以通过iframe将其嵌入到项目中。

## 控件对象
`kd.bos.form.control.IFrame`

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ----
 value | `value` | String |  | iframe内嵌网址的路径

## Java API
方法名 | 参数 | 说明
------- | -------  | ------
`setSrc` | String (第三方网页的地址: 即属性说明里的`value`) | 设置iframe的src

## 通用接口介绍及使用示例
- **设置iframe的src**
```java
IFrame iframe = this.getView().getControl("控件标识");
// 设置url
iframe.setSrc("value");
```

- **发送一个消息到iFrame中**

```java
IFrameMessage message = new IFrameMessage();

iframe.postMessage(message);
```

## 其他说明
- server端向第三方发送消息(pageId存在第三方页面的地址中, 由苍穹平台自动插入):
   1. server端调用iframe.postMessage方法(控件级方法), 触发苍穹前端postMessage_reduce方法;
   2. 前端将获取到的参数(苍穹后端和第三方页面约定的操作类型, 参数值, 第三方的页面地址)通过H5的postMessage再向第三方发送消息;
   3. 第三方需要在全局监听message方法, 用以接收父级页面传过来的值
   ```javascript   
    // 监听消息'message'，响应其他页面请求
    function addEventListenerOfMessage (cb) {
        // 为window注册message事件并绑定监听函数
        if (window.addEventListener) {
            window.removeEventListener('message', cb, false)
            window.addEventListener('message', cb, false)
        } else if (window.attachEvent) { // 兼容
            window.detachEvent('message', cb)
            window.attachEvent('message', cb)
        }
    }
   ```
- 第三方页面向苍穹server端通信:
   > 参数格式: 第三方页面使用H5的postMessage时, 第一个参数类型是对象, 对象里面的属性如下 


属性名 | 类型 | 默认值 | 说明  
------- | ------- | ------ | ------- 
pageId | string | '' | 可从第三方的页面地址(window.location.search)中获取
type | string | '' | 'invokeCustomEvent'-调用平台方法，往服务端发送自定义事件(eg: **window.postMessage({pageId:"xxxxx", type:'invokeCustomEvent', content:{foo:'foo',bar:'bar'}},'*')**); 'invokePlatformAction'-调用苍穹前端平台window上第三方挂接的函数(eg: **window.postMessage({pageId:"xxxxx", type:'invokePlatformAction', actionName:'函数名称', content:{foo:'foo',bar:'bar'}},'*')**)
content | any | null | 任意数据类型

   1. 第三方页面调用H5的postMessage方法, 向父级页面发送消息;
   2. 父级页面(苍穹前端平台)全局监听message方法, 获取相关参数(参数格式有要求, 如上);
   3. 取到参数后, 通过type调用苍穹前端平台的不同方法; 其中'invokeCustomEvent'是向苍穹server端发送消息.

- 之前的通信方式(pageId存在参数中, 第三方是通过Java插件的形式直接跟后端通信)保持不变, 且仍然可以使用.
