# 图片
>单据中需要显示一张图片时，可在平台提供的图片库选择或插件设置图片URL。

## 控件对象
`kd.bos.form.control.Image`

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:-------: | :-------: | :-------: | :-------: | -------
标识 | `imageKey` | string | | 从图片列表中选择一张图片
悬停图片 | `imageHoverKey` | string | | 从图片列表中选择一张图片当做鼠标悬停时展示的图片
允许点击 | `clickable` | boolean | `false` | 设置图片是否允许点击
上传按钮 | `iu` | boolean | `false` | 在图片允许点击时，设置图片是否允许上传
操作代码 | `opk` | string | | 图片点击时调用的操作（新增、删除、保存等）
上传配置 | `uconfig` | Map | | 在图片允许点击上传时，设置图片的上传配置
放大镜 | `sem` | boolean | `false` | 鼠标悬停时，局部放大图片细节
请求动画 | `controlLoading` | boolean | `false` | 在图片允许点击时，显示加载动画

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例
- **可点击时，插件范例**

```java
public class TestPlugin extends AbstractFormPlugin implements ClickListener {
  public void regiterListener(EventObject e){
    // 1.注册监听
    Image image = this.getView().getControl("控件标识");
    image.addClickListener(this);
  }

  public void click(EventObject e) {
   // 2. 响应方法，do something...
    Control control = (Control) evt.getSource();
    String key = control.getKey();
    if ("控件标识".equals(key)) {
      // do some cool thing...
    }
  }
}
```  

- **预览图片，插件范例**

```java
public class TestPlugin extends AbstractFormPlugin implements ClickListener {
  @Override
	public void entryRowDoubleClick(RowClickEvent evt) {	
		AbstractGrid grid = this.getView().getControl(ENTRYKEY);
		int rowIndex = grid.getEntryState().getFocusRow();
		Object url = this.getModel().getValue("url0", rowIndex);
		if (StringUtils.isNotBlank(url)) {
			String[] urls = { (String) url };
			this.getView().showPictureView(urls, 1);
		}
	}
}
```  

- **可设置鼠标悬停图片时显示的tips**

```java
Image image = this.getView().getControl("控件标识");
image.setTips("文字内容");
```  

- **支持徽标**

![示例](./image/badge.png ':size=450x130')

```java
// 新建一个badgeInfo对象
BadgeInfo info = new BadgeInfo();
info.setOverflowCount(99); // 显示数量上限，超过显示 99+
info.setCount(5); // 显示数量
info.setOffset(new String[]{"-5px", "-5px"}); // 显示位置[right，top]，默认是[0，0]
info.setDot(false); // 内容显示为小红点，默认是false

// 调用控件方法
Image image = this.getControl("imageap");
image.setBadgeInfo(info);
```

!> 和图片字段的区别
- 图片字段：有业务语义，在数据库表中有对应列。
- 图片控件：没有业务语义，只是单纯显示一张图片。

