# 地图控件
>展示地图

## 控件对象
`kd.bos.ext.form.control.MapControl`

## 视觉展示

- **默认**

![map_control_normal.png](./image/map_control_normal.png) 

- **用户搜索`关键字`后的效果** 

![map_control_search.png](./image/map_control_search.png) 

- **后端插件指定地图定位到某个位置的效果** 

![map_control_selectAddress.png](./image/map_control_selectAddress.png) 

## 控件属性说明

属性名 | 属性ID | 类型 | 说明 
---- | :----: | :----: | ---- 
显示搜索框 | `showSearchBox` | bool | 默认true，是否显示地图左上角的搜索框
标记是否可拖动 | `canMarkerDraggble` | bool | 默认false，属性打开后可用鼠标拖动地图上的位置标记
经纬度标记 | `selectPoint` | map | 根据经纬度在地图上标记一个点
地址标记 | `selectAddress` | map | 根据一个地址信息在地图上标记一个点

控件通用属性请参考=>[传送门](advanced/controlProperties/)

## 服务端API

方法名 | 功能说明 | 参数结构示例
---- | ---- | ---- 
selectPoint | 设置前端地图控件选中某个节点(节点包含具体信息) | {"title":"金蝶软件","lng":113.96624,"lat":22.576339,"address":"南山科技园科技南12路","city":"深圳市","province":"广东省”}
selectAddress | 设置前端地图标记某个地址,以节点形式 | {detailAddress:广东省深圳市南山区金蝶大厦A座}

## 功能说明

功能名 | 实现方式 | 说明 
---- | ---- | ---- 
搜索 |  |地图左上角有搜索框，使用该搜索框可以根据用户录入关键字搜索出多个结果，鼠标点击单个结果地图会自定定位并标记
根据经纬度定位 | 控件方法`selectPoint` | 后端插件调用该方法传送经纬度信息，前端自动根据该信息在地图上定位并标记
根据具体地址定位 | 控件方法`selectAddress` |后端插件调用该方法传送地址信息，前端自动根据该信息在地图上定位并标记

## 前端回传定位信息
- **点击地图上某个标记节点，前端会自动往后端发送该标记信息，数据示例如下:**
```javascript
[{"key":"mapcontrolap","methodName":"select","args":[{
"latitude":22.53979005313571,
"longitude":113.96133709676266,
"province":"广东省",
"city":"深圳市",
"district":"南山区",
"name":"","address":"金蝶软件园内,金蝶大厦附近25米",
"addressdetail":"广东省深圳市南山区高新南七道 金蝶软件园","origin":"广东省深圳市南山区金蝶大厦A座"}],
"postData":[{},[]]}]
```


## 定位数据结构示例
```javascript
{
    "title":"金蝶软件(民德路)",
    "point":{"lng":114.04663,"lat":22.624036},
    "address":"民治大道317东边商务大楼3059室",
    "city":"深圳市",
    "province":"广东省"
}
```
## 插件示例

- **插件代码**
 
```java
public void do() {
    // 设置显示隐藏搜索框（通用u指令）
    Map<String, Object> dataMap = new HashMap<>();
    dataMap.put("showSearchBox", false); //属性名称+值
    dataMap.put("k", "控件id");
    IClientViewProxy proxy = this.getView().getService(IClientViewProxy.class);
    proxy.addAction(ClientActions.updateControlStates, dataMap);

    // 设置是否可拖动地图标记
    Map<String, Object> dataMap = new HashMap<>();
    dataMap.put("canMarkerDraggble", true); //属性名称,值
    dataMap.put("k", "控件id");
    IClientViewProxy proxy = this.getView().getService(IClientViewProxy.class);
    proxy.addAction(ClientActions.updateControlStates, dataMap);

    // 设置地图根据经纬度标记某个位置
    MapControl mapCtl = this.getView().getControl("控件id");
    mapCtl.selectPoint(MapSelectPointOption对象参数示例)  //MapSelectPointOption：经纬度封装对象

    // 设置地图根据地址标记某个位置
    MapControl mapCtl = this.getView().getControl("控件id");
    mapCtl.selectAddress("广东省深圳市南山区科技园金蝶软件");
}
```

