# 评分控件
> 移动端评分控件用于对事物进行评分的操作，也可以用于对评价进行展示，能够让用户快捷地对某一事物进行反馈。评分控件目前支持标准评价和情感评价两种模式。

## 控件对象
`kd.bos.ext.form.control.Rate`

## 视觉展示
### 移动端
![倒计时](./image/rate.gif)

### PC端
![倒计时](./image/rate1.png)
![倒计时](./image/rate2.png)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ---- 
评价类型 | `rateType` | String | `normal` | 控制评价类型,（`normal`： 标准评价，`emotional`：情感化评价）
总分 | `totalScore` | Number | `5` | 设置控件总分（控件恒定5个图标，总分用于控制各个图标对应的分值，如总分100分，那么点击第一个图标时当前分值为20，点击第二个时为40...）
显示文本类型 | `showTextType` | Number | `0` | 控制在图标后面展示的文本类型，`0`表示不显示，`1`表示分值，`2`表示描述文本
图标大小 | `iconSize` | Number | `18` | 控制图标尺寸，单位px，最小可设置为12，最大36
数量 | `starsNum` | Number | `5` | 设置标准模式时星星的数量
允许半选 | `allowHalf` | Boolean | `false` | 设置标准模式时星星是否开启半选状态

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件 API
接口名称 | 参数 | 说明  
------- | ------- | -------  
`setRateScore` | `Number` | 设置当前分值
`setRateTexts` | `List<String>` | 设置分值对应的描述文本（需先设置显示文本类型为`文本`）

## 插件示例
- **插件设置评分值、插件设置分值对应的描述文本、监听前端变更的的评分值**

```java
import kd.bos.form.control.Rate;
import kd.bos.form.control.events.RateEvent;
import kd.bos.form.control.events.RateListener;
import kd.bos.form.plugin.AbstractFormPlugin;

import java.util.Arrays;
import java.util.EventObject;

public class TestPlugin extends AbstractFormPlugin implements RateListener {

	@Override
	public void registerListener(EventObject e) {
		Rate rate = this.getControl("rate");
		rate.addRateListener(this);
	}

	@Override
	public void afterCreateNewData(EventObject e) {
		Rate rate = this.getControl("rate");
		
		// 插件设置评分值
		rate.setRateScore(1);
		
		// 插件设置分值对应的描述文本
		rate.setRateTexts(Arrays.asList("E级", "D级", "C级", "B级", "A级"));
	}

	/**
	 * 监听前端变更的的评分值
	 *
	 * @param rateEvent
	 */
	@Override
	public void update(RateEvent rateEvent) {
		Number rateScore = rateEvent.getRateScore();
	}
}
```  
