# 滑动输入条
> 滑动输入条控件是一个提供以滑动的方式在数值区间快捷选择数值或数值区间的控件，每一次滑动的数值增长（减少）量以配置的“步长”值为准。同时支持通过直接点击滑动条上的任一位置快速选择数值的功能。

## 控件对象
`kd.bos.ext.form.control.Slider`

## 视觉展示

![倒计时](./image/slider.gif)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ---- 
是否双数值 | `isRange` | Boolean | `false` | 控制是否双数值选择模式
数值范围 | `dataScope` | String |  | 设置数据可选范围，例：`[0,99]`
步进值 | `stepValue` | Number | `1` | 控制每一次滑动的数值增长（减少）量，注：'(数值范围最大值 - 数值范围最小值) / 步进值' 的结果必须为正整数
缺省值 | `defValue` | String | 0 | 缺省值默认是0，如果使用过程中没有设置相应的值则用该缺省值（例：`[0,50]`，双数值模式时分别表示两个滑块的缺省值分别对应为`0`跟`50`，单数值模式时也可以输入`50`，表示缺省值为50）
展示当前值 | `showCurrentValue` | Boolean | `false` | 是否展示值，为false则表示不展示值，只在滑动时展示，如上视觉展示图  

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件 API
接口名称 | 参数 | 说明  
------- | ------- | -------  
`setSliderValue` | `Number,Number` | 设置滑动输入条的值，双数值模式时直接取参数的两个值作为当前已选数值区间，单数值模式时取参数的第二个元素作为当前已选数值
`setSliderMarks` | `Map<String, Map<String, Object>>` | 滑动输入条设置刻度标记，参数示例：[{"10": {label: "数值对应文本", style: {}}}]，`10`表示要设置刻度的数值，`label`表示数值对应文本，`style`表示文本对应的样式

## 插件示例
- **插件更新滑动输入条的值、插件更新滑动输入条刻度、监听前端变更滑动输入条的值**  

```java
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.form.control.Slider;
import kd.bos.form.control.events.SliderEvent;
import kd.bos.form.control.events.SliderListener;
import kd.bos.form.plugin.AbstractFormPlugin;

import java.util.EventObject;
import java.util.Map;

public class TestPlugin extends AbstractFormPlugin implements SliderListener {

	@Override
	public void registerListener(EventObject e) {
		Slider slider = this.getControl("slider");
		slider.addSliderListener(this);
	}

	@Override
	public void afterCreateNewData(EventObject e) {
	    Slider slider = this.getControl("slider");

		// 插件更新滑动输入条的值
		slider.setSliderValue(0, 100);

		// 插件更新刻度
		slider.setSliderMarks(SerializationUtils.fromJsonString
				("{\"20\":{\"style\":{\"color\":\"#F00\"},\"label\": \"38X\"}}", Map.class));
	}

	/**
	 * 监听前端变更滑动输入条的值
	 *
	 * @param sliderEvent
	 */
	@Override
	public void update(SliderEvent sliderEvent) {
		Number[] value = sliderEvent.getValue();
		Number leftValue = value[0];
		Number rightValue = value[1];
	}
}
```  
