# 步骤条控件
> 移动端步骤条控件用于显示一个任务或事件的进度，也用于一步步地指引用户完成一系列操作。每个步骤项都会随着当前进度改变对应的步骤状态，可以清晰地向用户展示已完成、进行中、未完成和异常状态的步骤。

## 控件对象
`kd.bos.ext.form.control.Steps`

## 视觉展示

![倒计时](./image/steps.gif)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ---- 
方向 | `stepsDirection` | String | `horizontal` | 步骤条方向,（`horizontal`： 水平，`vertical`：垂直）
步骤条类型 | `stepsType` | String | `normal` | 步骤条类型,（`normal`： 标准步骤，`number`：数字步骤）
步骤长度 | `stepsLength` | Number | `4` | 步骤长度，用于快速配置指定长度的步骤条，会被“步骤项配置”属性覆盖（必须为大于0的整数，步骤条方向设置为水平方向时最大4，纵向无限制）
步骤项配置 | `stepsOptions` | Array |  | 用于配置步骤条各个步骤项的标题以及描述信息，当该属性有配置时，步骤条实际长度以该属性长度为准
标题行数 | `titleLine` | Number | `0` | 控制步骤项标题文本最大显示行数，超出部分将以省略号表示，为0表示不限制
描述行数 | `descriptionLine` | Number | `0` | 控制步骤项描述文本最大显示行数，超出部分将以省略号表示，为0表示不限制

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件 API
接口名称 | 参数 | 说明  
------- | ------- | -------  
`setCurrentStep` | `Map<String, Object>` | 设置当前步骤及对应的状态
`setStepsOptions` | `List<StepsOption>` | 设置整个步骤条的配置

## 插件示例
- **设置整个步骤条的配置、设置当前步骤及对应的状态**

```java
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.form.control.Steps;
import kd.bos.form.control.StepsOption;
import kd.bos.form.plugin.AbstractFormPlugin;

import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TestPlugin extends AbstractFormPlugin {

	@Override
	public void afterCreateNewData(EventObject e) {
		Steps steps = this.getControl("steps");

		// 获取设计时的步骤条设置
		List<StepsOption> stepsOptions = steps.getStepsOptions();
		stepsOptions.clear();

		StepsOption stepsOption0 = new StepsOption();
		stepsOption0.setTitle(new LocaleString("First"));
		stepsOption0.setDescription(new LocaleString("First Step"));
		stepsOption0.setStatus(Steps.FINISH);   // 完成
		stepsOptions.add(stepsOption0);

		StepsOption stepsOption1 = new StepsOption();
		stepsOption1.setTitle(new LocaleString("Second"));
		stepsOption1.setDescription(new LocaleString("Second Step"));
		
		stepsOption1.setStatus(Steps.PROCESS);  // 进行中
		stepsOption1.setStatus(Steps.FINISH);   // 完成
		stepsOption1.setStatus(Steps.ERROR);    // 异常
		stepsOptions.add(stepsOption1);
		
		StepsOption stepsOption2 = new StepsOption();
		stepsOption2.setTitle(new LocaleString("Third"));
		stepsOption2.setDescription(new LocaleString("Third Step"));
		stepsOptions.add(stepsOption2);
		// 更新步骤条设置
		steps.setClientStepsOptions(stepsOptions);

		// 设置当前节点
		Map<String, Object> currentStepMap = new HashMap<>();
		currentStepMap.put("currentStep", 1);
		currentStepMap.put("currentStatus", Steps.ERROR);
		// 更新当前节点
		steps.setCurrentStep(currentStepMap);
	}
}
```  
