# 时间轴控件
> 时间轴控件主要用于垂直展示时间流信息，按时间顺序逐步展示相应信息。  

## 控件对象
`kd.bos.ext.form.control.Timeline`

## 视觉展示

![倒计时](./image/pctimeline.png)
![倒计时](./image/mobiletimeline.png)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
:----: | :----: | :----: | :----: | ---- 
时间轴位置 | `axisPositon` | String | `left` | 时间轴位置,（`left`： 左侧，`right`：右侧）
时间节点 | `timelineOptions` | Array |  | 用于配置时间轴各个时间节点的标题以及描述信息
标题行数 | `titleLine` | Number | `0` | 控制步骤项标题文本最大显示行数，超出部分将以省略号表示，为0表示不限制
描述行数 | `descriptionLine` | Number | `0` | 控制步骤项描述文本最大显示行数，超出部分将以省略号表示，为0表示不限制

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件 API
接口名称 | 参数 | 说明  
------- | ------- | -------  
`setTimelineOptions` | `List<TimelineOption>` | 设置整个时间轴时间节点的配置

## 插件示例
- **插件设置时间轴各个节点配置**

```java
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.form.control.Timeline;
import kd.bos.form.control.TimelineContentOption;
import kd.bos.form.control.TimelineLabelOption;
import kd.bos.form.control.TimelineOption;
import kd.bos.form.plugin.AbstractFormPlugin;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

public class TestPlugin extends AbstractFormPlugin {

	@Override
	public void afterCreateNewData(EventObject e) {
		Timeline timeline = this.getView().getControl("timeline");

		// 获取设计时的时间轴选项
		List<TimelineOption> timelineOptions = timeline.getTimelineOptions();

		TimelineOption timelineOption = timelineOptions.get(0);

		// 时间轴内容配置
		TimelineContentOption timelineContentOption = timelineOption.getContent();

		// 获取内容详情
		LocaleString timelineContentDescription = timelineContentOption.getDescription();
		// 获取内容标题
		LocaleString timelineContentTitle = timelineContentOption.getTitle();

		// 时间轴标签配置
		TimelineLabelOption timelineLabelOption = timelineOption.getLabel();

		// 获取标签详情
		LocaleString timelineLabelDescription = timelineLabelOption.getDescription();
		// 获取标签标题
		LocaleString timelineLabelTitle = timelineLabelOption.getTitle();


		// 新的时间轴选项配置
		List<TimelineOption> newTimelineOptions = new ArrayList<>(4);

		TimelineOption newTimelineOption = new TimelineOption();

		// 设置内容标题和内容详情
		newTimelineOption.setContent(new TimelineContentOption(new LocaleString("ContentTitle"), new LocaleString("Content")));
		// 设置标签标题和标签内容
		newTimelineOption.setLabel(new TimelineLabelOption(new LocaleString("LabelTitle"), new LocaleString("LabelContent")));

		newTimelineOptions.add(newTimelineOption);

		// 设置新的时间轴配置，发送给前端
		timeline.setClientTimelineOptions(newTimelineOptions);
	}
}
```  
