# 日期范围字段
>日期范围选择控件。

## 控件对象
`kd.bos.form.field.DateRangeEdit`

## 视觉展示

![daterange_default.jpg](./image/daterange_default.jpg)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ---- 
开始标识 | startDateFieldKey | string |  | 开始日期标识
结束标识 | startDateFieldName | string |  | 结束日期标识
开始字段名 | endDateFieldKey |  string |  | 开始日期数据库字段名
结束字段名 | endDateFieldName |  string |  | 结束日期数据库字段名
区域类型 | `regiontype` | number | `1` | 控制长日期字段显示为短日期格式或者长日期格式，1为短日期，2为长日期

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例

- **演示如何存取字段值，只演示存取单据头字段值**

```java
private void demoFieldValue(){

	// 需要通过如下方式，获取日期范围字段，开始、结束属性对象的标识
	DateRangeEdit headFieldEdit = this.getView().getControl(KEY_HEADDATERANGE);
	String key_headdatestart = headFieldEdit.getStartDateFieldKey();
	String key_headdateend = headFieldEdit.getEndDateFieldKey();

	// 获取单据头日期范围，开始时间、结束时间
	Date headdatestart = (Date) this.getModel().getValue(key_headdatestart);
	Date headdateend = (Date) this.getModel().getValue(key_headdateend);
	
	// 赋值
	this.getModel().setValue(key_headdatestart, headdatestart);
	this.getModel().setValue(key_headdateend, headdateend);
	
}
```

beforePropertyChanged	修改字段值之前触发
例外：界面数据初始时，不触发此事件，即在afterCreateNewData事件中，修改字段值，不会触发此事件

```java
package kd.bos.plugin.sample.dynamicform.pcform.field.template;

import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.plugin.AbstractFormPlugin;

public class DateRangeBeforePropertyChanged extends AbstractFormPlugin {

	private final static String KEY_STARTDATE1 = "startdate1";
	private final static String KEY_ENDDATE1 = "enddate1";
	
	@Override
	public void beforePropertyChanged(PropertyChangedArgs e) {
		String fieldKey = e.getProperty().getName();
		if (StringUtils.equals(KEY_STARTDATE1, fieldKey)){
			// 开始日期值改变
			// TODO 在此添加业务逻辑
		}
		else if (StringUtils.equals(KEY_ENDDATE1, fieldKey)){
			// 结束日期值改变
			// TODO 在此添加业务逻辑
		}
	}
}
```

propertyChanged	修改字段值之后触发
例外：界面数据初始时，不触发此事件，即在afterCreateNewData事件中，修改字段值，不会触发此事件

```java
package kd.bos.plugin.sample.dynamicform.pcform.field.template;

import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.plugin.AbstractFormPlugin;

public class DateRangePropertyChanged extends AbstractFormPlugin{

	private final static String KEY_STARTDATE1 = "startdate1";
	private final static String KEY_ENDDATE1 = "enddate1";
	
	@Override
	public void propertyChanged(PropertyChangedArgs e) {
		String fieldKey = e.getProperty().getName();
		if (StringUtils.equals(KEY_STARTDATE1, fieldKey)){
			// 开始日期值改变
			// TODO 在此添加业务逻辑
		}
		else if (StringUtils.equals(KEY_ENDDATE1, fieldKey)){
			// 结束日期值改变
			// TODO 在此添加业务逻辑
		}
	}
}
```

