# 长日期字段
>长日期选择控件。

## 控件对象
`kd.bos.form.field.DateTimeEdit`

## 视觉展示

- **默认**

![datetime_default.jpg](./image/datetime_default.jpg)

- **展开时间选择**

![datetime_open.jpg](./image/datetime_open.jpg)

- **月份选择（掩码设置YYYY-MM）**

![datetime_month.jpg](./image/datetime_month.jpg)

- **年份选择（掩码设置YYYY）**

![datetime_year.jpg](./image/datetime_year.jpg)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ---- 
掩码 | `mask` | string |  | 设置日期控件的交互形态，可以配置选择年月日
格式化字符串 | `df` | string |  | 设置日期控件编辑完成后的显示格式
可选范围 |  |  |  | 设置日期控件开始时间和结束时间，选择日期时只能在这个范围内选择
时区转换 | `TimeZoneTransType` | number | `服务器时区` | 切换时区，长日期字段控制时区信息regiontype
区域类型 | `regiontype` | number | `2` | 控制长日期字段显示为短日期格式或者长日期格式，1为短日期，2为长日期
为空提示信息 | `emptytip` | Map | | 当字段内容为空时，输入框内显示的信息

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例

- **设置长日期**

```java
package kd.bos.plugin.sample.dynamicform.pcform.field.bizcase;

import java.util.Date;

import kd.bos.entity.datamodel.IDataModel;
import kd.bos.form.field.DateTimeEdit;
import kd.bos.form.plugin.AbstractFormPlugin;

public class DateTimeEditSample extends AbstractFormPlugin {

	private final static String KEY_ENTRYENTITY = "entryentity";
	
	private final static String KEY_DATETIMEFIELD1 = "datetimefield";
	private final static String KEY_DATETIMEFIELD2 = "datetimefield";
	private final static String KEY_DATETIMEFIELD3 = "datetimefield";
	
	/**
	 * 演示如何获取、设置长日期字段值
	 * 
	 * @param entryRow 单据体行号
	 * @param subEntryRow 子单据体行号
	 */
	private void demoFieldValue(int entryRow, int subEntryRow){
		
		IDataModel dataModel = this.getModel();
		
		// 存取单据头字段值
		Date headdatetime = (Date)dataModel.getValue(KEY_DATETIMEFIELD1);
		dataModel.setValue(KEY_DATETIMEFIELD1, headdatetime);
		
		// 存取单据体字段值
		Date entrydatetime = (Date)dataModel.getValue(KEY_DATETIMEFIELD2, entryRow);
		dataModel.setValue(KEY_DATETIMEFIELD2, entrydatetime, entryRow);
		
		// 存取子单据头字段值，必须先设置父单据体行号
		dataModel.setEntryCurrentRowIndex(KEY_ENTRYENTITY, entryRow);
		Date subentrytime = (Date)dataModel.getValue(KEY_DATETIMEFIELD3, subEntryRow);
		dataModel.setValue(KEY_DATETIMEFIELD3, subentrytime, subEntryRow);
	}
}
```