# 文本字段
>用户需要录入数据时使用，通过鼠标或键盘输入内容，是最基础的表单域包装。

?> 结合 **字段布局面板** 控件使用是最佳实践。

## 控件对象
`kd.bos.form.field.TextEdit`

## 视觉展示
风格 | 垂直 | 水平  
------- | ------- | -------  
基本 | ![basic_v.png](./image/basic_v.png) | ![basic_v.png](./image/basic_h.png)  
禁用 | ![basic_v.png](./image/disable_v.png) | ![basic_v.png](./image/disable_h.png)  
有跟随按钮 | ![basic_v.png](./image/btn_v.png) | ![basic_v.png](./image/btn_h.png)  
显示Tips |![tips_v.png](./image/tips_v.png) |![tips_h.png](./image/tips_h.png)
必录 | ![basic_v.png](./image/mustinput_v.png)| ![mustinput_h.png](./image/mustinput_h.png)
边框 | ![border_v.png](./image/border_v.png)| ![border_h.png](./image/border_h.png)

## 属性说明
### 字段通用属性  :id=fieldsx
字段通用属性 | 属性ID | 类型 | 默认值 | 说明 
:---: | :---: | :---: | :---: | --- 
标题宽度 | `lw` | Map | | 可以设置字段标题占的宽度大小
显示标题 | `sti` | boolean | `true` | 设置字段是否显示标题
标题对齐方式 | `text-align` | string | `默认` | 设置标题对齐方式
标题位置 | `lDir` | string | `垂直` | 设置标题位置
字段风格 | `fieldstyle` | number | `缺省` | 设置字段的显示风格
即时触发值更新 | `fu` | boolean | `false` | 设置字段在失去焦点时是否立即发送值更新指令到后端
显示tips | `tips` | Map | | 配置提示信息，类型为`表单`时，<u>*需要表单设置固定宽高，否则表单显示位置会错乱*</u>
字段名 | | string | | 对应数据库列名，动态表单没有该属性
必录 | `mi` | boolean | `false` | 设置字段是否为必须录入字段
缺省值 | | | | 设置字段的默认初始值

### 业务属性  :id=textsx
业务属性 | 属性ID | 类型 | 默认值 | 说明 
:---: | :---: | :---: | :---: | --- 
最大长度 | `maxlength` | number | `50` | 字段允许输入的最大长度范围，超出输入无效
最小长度 | `minlength` | number | `0` | 字段允许输入的最小长度范围
编辑风格 | `showEditButton/eb` | boolean | `文本框编辑` | 设置文本类型字段的编辑风格，showEditButton设置是否显示编辑按钮；eb设置文本输入框是否可编辑
按钮图标 | `fontClass` | string | | 在文本类型字段为按钮编辑风格时，可以设置按钮的图标
为空提示信息 | `emptytip` | Map | | 当字段内容为空时，输入框内显示的信息
锁定为空提示信息 | `lockedemptytip` | Map | | 当字段锁定且内容为空时，输入框内显示的信息
内容显示为密码 | `type` | string | `false` | 输入内容加密，显示为*号
加密存储 | | boolean| `false` | 设置文本是否加密存储
格式化 | `tfmt` | string |  | 格式化有两种模式 : 默认模式和银行卡模式，银行卡模式只允许输入数字，每4位有空格分割

### 样式属性  :id=stylesx
样式属性 | 属性ID | 类型 | 默认值 | 说明 
:---: | :---: | :---: | :---: | --- 
占整行 | `fullLine`| boolean | `false` | 放在`字段布局面板`中使用时有该属性，设置字段是否占整行显示

?> 控件通用属性[请参考](advanced/controlProperties/)

### 即时触发值更新 :id=fu
该属性打开后，文本数据录入完成鼠标失去焦点时，会立即将输入的内容发送到服务端。
单据插件需要实现`propertyChanged`方法可处理录入数据。
```java
public class TestPlugin extends AbstractFormPlugin {
  public void propertyChanged(PropertyChangedArgs e) {
    // do something...
  }
}
```  
### 编辑风格  :id=editstyle
该属性有三个选项，选择`按钮编辑`或者`按钮+文本编辑`后，出现`按钮图标`属性![btnicon.png](./image/btnicon.png)，可修改跟随按钮图标。
点击按钮，发送`click`事件到服务端插件。
```java
// 插件监听click事件
public class TestPlugin extends AbstractFormPlugin implements ClickListener {
  public void regiterListener(EventObject e){
    // 1.注册监听
    TextEdit edit = this.getView().getControl("字段标识");
    edit.addButtonClickListener(this);
  }
  
  public void click(EventObject e) {
   // 2. 响应方法，do something...
  }
}
```  

## 插件示例  :id=fieldplug
- **文本框值操作**

```java
// 获取文本字段值
String text = this.getModel().getValue("字段标识");

// 设置文本字段值
this.getModel().setValue("字段标识", value);
```  
- **可见性**

```java
// 文本字段隐藏
this.getView().setVisible(false, "字段标识");

// 文本字段显示，注意：设计器中出了`初始可见`之外的可见性状态无法通过插件代码修改
this.getView().setVisible(true, "字段标识");
```  
- **锁定性**

```java
// 锁定文本字段
this.getView().setEnable(false, "字段标识");

// 解锁，注意：设计器中的`锁定性`属性锁定的无法通过插件代码解锁
this.getView().setEnable(true, "字段标识");
```  
- **设置字段必录**

```java
TextEdit edit = this.getView().getControl("字段标识");
edit.setMustInput(true);
```  
- **设置tips不显示**

```java
TextEdit edit = this.getView().getControl("字段标识");
edit.hideTips();
```  

- **动态添加文本类Tips**

```java
TextEdit textEdit = this.getControl("字段标识");
Tips tips = new Tips();

// 组织tips文本类型对象数据
// 组织内容，这里有配置链接信息，其中 #{0}为链接占位符，Tips.Warp换行符，Tips.Tab缩进符
StringBuffer content = new StringBuffer();

// 链接站位索引号
int index = 0;
content.append("换行").append(Tips.Warp).append("插入链接占位符").append(Tips.insertLinkPlaceholder(index)).append("缩进").append(Tips.Tab).append("ccc");

// 创建连接数据对象
List<Map<String, Object>> linkData = Tips.createLinkData(new TipsLink(index, new LocalString("下一步"), "next"));


// 添加的tips类型为 文本，类型为表单的配置方法见分割线下方
tips.setType(Tips.Text);
tips.setTitle(new LocalString("标题"));
tips.setContent(new LocalString(content.toString()));
tips.setLink(true); // 有配置链接信息为true，否则 false
tips.setLinkData(linkData);

// 添加tips对象
textEdit.addTips(tips);

```  

- **动态添加表单类Tips**

```java
TextEdit textEdit = this.getControl("字段标识");
Tips tips = new Tips();

// 配置tips类型为表单
tips.setType(Tips.Bill);
tips.setBillno("表单id"); // 类型为表单时，表单id不能为空

// 添加tips对象
textEdit.addTips(tips);

```  

