# 轮播容器
> 在一定空间内循环展示多个信息；  
控件类型不仅仅是图片，还可以是`按钮`、`标签`和`矢量图标`。


## 控件对象
`kd.bos.ext.form.control.MessageCarouselContainer`

## 视觉展示
![carouseshow.gif](./image/carouseshow.gif)

## 属性说明
属性名 | 属性ID | 类型 |默认值| 说明 
---- | :----: | :----: | :---- : | ---- 
轮播数量 | `displayNum` | number |`1`|在面板显示的子项数目
轮播方向 | `playDirection` | string |`由下往上`|轮播内容切换的方向，有四个值：由下往上、由上往下、由左往右、由右往左
速度 | `speed` | number | `1000`|循环播放时项目之间的延迟时间，单位ms
延迟 | `delay` | number |`1000`|多久后轮播下一个子项，也就是轮播项目停留的时间，单位ms

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例
- **事件监听**


```java
// 实现「ListboxClickListener」接口
public class TestPlugin extends AbstractFormPlugin implements ItemClickListener, ClickListner {
  public void registerListener(EventObject event) {

    // 注册监听
    MessageCarouselContainer mcc = this.getView().getControl("控件标识");
    mcc.addItemClickListener(this);
    mcc.addClickListener(this);
  }

  // 一行中的子项点击事件 发生之前，可以取消事件继续
  public void beforeItemClick(BeforeItemClickEvnet event) {
    // do something...
  }
  
  // 一行中的子项点击事件
  public void itemClick(BeforeItemClickEvnet event) {
    // do something...
  }
  
  // 行点击事件 发生之前，可以取消事件继续
  public void beforeClick(BeforeClickEvnet event) {
    // do something...
  }
  
  // 行点击事件
  public void click(BeforeClickEvnet event) {
    // do something...
  }
}
```  

- **添加数据**

```java
/**
 * 向轮播容器中动态添加三条数据
 * 包含 图片、标签控件。
 */
MessageCarouselContainer mcc = this.getControl("控件标识");
List<Map<String, Object>> data = new ArrayList<>();

// 图片控件url数组
String[] urls = new String[] {"url1", "url2", "url3"};

// 标签控件显示内容数组
String[] texts = new String[] {"t1", "t2", "t3"};

// 放内容map
Map<String, Object> propsMap = null;

// 控件map，key为控件类型
Map<String, Object> controlsMap = null;

for (int i = 0; i < urls.length; i++;) {
  propsMap = new HashMap<>();
  controlsMap = new HashMap<>();
  
  propsMap.put("text", texts[i]);
  controlsMap.put("labelap", propsMap);
  
  propsMap = new HashMap<>();
  propsMap.put("imageKey", urls[i]);
  controlsMap.put("imageap", propsMap);
  
  data.add(controlsMap);
}

mcc.setData(data);

```  


## 插件API
方法名 | 参数 | 说明  
------- | ------- | -------  
`setData` | `List<Map<String, Object>>` | 添加轮播数据  
