# Flex面板
>具有弹性布局特性的容器控件。

## 控件对象
`kd.bos.form.container.Container`

## 为什么要布局
表单好比一套房子，其中每个房间的大小、方向、家具排列方式等均可通过`Flex面板`控制。

## 如何优雅地开发一张漂亮的表单
### 黄金三步曲
1. 放置合适的容器控件
2. 放置其他控件
3. 设置容器布局

## 如何实现容器的快速布局
以`Flex面板`为例，表单中放了一个`Flex面板`，面板中放了两个字段。

选中`Flex面板`，切换到`样式`标签，通过![快速布局](./image/onestep.png)功能，可一键设置容器布局。如下：

![快速布局动图](./image/onestep.gif)

**如有更复杂的布局需求，可点击`快速布局`下方的`进入高级设置`**

![切换高级设置动图](./image/advsetting.gif)

## 控件占满容器的高度

进入容器布局`高级设置`，设置成`水平布局`，水平方向对齐方式`默认`，垂直方向对齐方式`充满(stretch)`。

!> 注意：强烈不建议通过设置控件的`高度`或者`宽度` 为`100%`来占满容器，这个条件生效的前提是其继承来的父容器设置了固定的宽高，否则会出现各种奇怪的兼容性问题，推荐通过弹性布局来实现。

![充满高度](./image/stretch_v.gif)
## 控件占满容器宽度
选中容器，进入容器布局`高级设置`，设置成`垂直布局`，水平方向对齐方式`充满(stretch)`，垂直方向对齐方式`默认`。

![充满宽度](./image/stretch_h.gif)

## 控件占满整个容器
1. 选中容器，进入容器布局`高级设置`，设置成`垂直布局`，水平方向对齐方式`充满(stretch)`，垂直方向对齐方式`默认`。
2. 选中控件，将`扩展比率`设置成 `1`。
![充满整个容器](./image/stretch.gif)

## 控件固定在右边显示
选中容器，进入容器高级布局，设置成`水平布局`，水平方向对齐方式`终点对齐(flex-end)`

![right.gif](./image/right.gif)

> 其他问题点击 [常用布局](https://docs.qq.com/doc/BqI21X2yZIht12YoSk1L4qhy3Pmkf732HiWR4ifBxK3x7kCy0IQmKC2Cjyb92kjUHU05J2tP0z2BvX4RsXCl2) 了解更多。

----

## 属性介绍
业务属性名 | 属性ID | 类型 | 默认值 | 说明  
------- | :----: | :----: | :----: | -------  
可折叠 | `collapsible` | Boolean | false | 设置面板是否可折叠  
默认折叠 | `defaultcollapse` | Boolean | false | 设置面板默认的折叠状态
隐藏折叠头 | `hidetitle` | Boolean | false | 设置面板开启折叠时头部的显示与隐藏（mobile端）
折叠头背景色 | `tbc` | Sting | '' | 设置面板开启折叠时头部的背景颜色（mobile端）
折叠摘要 | `cf` | Array | | 面板折叠情况下显示已配置的折叠摘要信息  
允许点击 | `clickable` | Boolean | fasle | 点击面板发送click请求
显示Tips | `tips` | Object | | 可折叠情况下显示配置的Tips信息  
允许全屏 | `maximize` | Bollean | false | 允许flex面板全屏

<br/>

样式属性名 | 属性ID | 类型 | 默认值 | 说明  
------- | :----: | :----: | :----: | -------  
扩展比率 | `gr` | Number | 1 | 父容器空间剩余时，本身按扩展比例扩展，可扩展`1`，不可扩展`0`
压缩比率 | `sk` | Number | 1 | 父容器空间不够时，本身按压缩比例压缩，可压缩`1`，不可压缩`0`
圆角半径 | `rd` | String | | 设置控件圆角半径
侧轴对齐 | `as` | String | default | 设置子项在容器侧轴方向上的对齐方式，该属性对容器子项生效
堆叠顺序 | `zIndex` | Number | 0 | 设置控件的堆叠顺序
溢出 | `of` | String | auto | 容器空间不够时的呈现效果
悬停阴影 | `shadow` | Boolean | false | 设置鼠标悬停是否显示阴影

> 详细可点击 [Flex语法](http://www.ruanyifeng.com/blog/2015/07/flex-grammar.html) 和 [Flex样例](http://www.ruanyifeng.com/blog/2015/07/flex-examples.html) 了解详情。

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例
- **插件中动态往容器中添加子控件**

```java
// 向flex面板动态添加按钮控件
Container flexPanel = this.getControl("Flex面板标识");

ButtonAp button = new ButtonAp();
button.setKey("testkey");
button.setName(new LocalString("动态添加按钮"));

container.addControls(Arrays.asList(button.createControl()));
```  
- **动态往容器中插入子控件**

```java
Container flexPanel = this.getView().getControl("Flex面板标识");

ButtonAp button = new ButtonAp();
button.setKey("testkey");
button.setName(new LocalString("动态添加按钮"));

flexPanel.insertControls(index, Arrays.asList(button.createControl()));
```  
- **动态删除容器中的子控件**

```java
Container flexPanel = this.getView().getControl("Flex面板标识");
String[] keys = new String[]{"控件标识1", "控件标识2"};
// ...
flexPanel.deleteControls(keys);
```  
- **设置Flex面板折叠/展开**

```java
Container flexPanel = this.getView().getControl("Flex面板标识");
// 设置折叠
flexPanel.setCollapse(true);

// 展开
flexPanel.setCollapse(false);
```  
- **动态设置容器背景图片**

```java
Container flexPanel = this.getView().getControl("Flex面板标识");
String url = "";
flexPanel.setBackgroundImg(url);
```  

- **动态创建一个Flex面板**

```java
FlexPanelAp flex = new FlexPanelAp();
flex.setKey("flexpanelap");
// 设置一系列属性
// ...

// 将面板添加到父容器中
container.addControls(Arrays.asList(flex.createControl()));
```  