# 页签控件
>选项卡切换组件。

## 控件对象
`kd.bos.form.container.Tab`

## 视觉展示

- **默认风格** 

![tabcontrol_default.png](./image/tabcontrol_default.png) 

- **格子风格** 

![tabcontrol_grid.png](./image/tabcontrol_grid.png) 

- **简洁风格** 

![tabcontrol_laconic.png](./image/tabcontrol_laconic.png) 

- **迷你风格** 

![tabcontrol_mini.png](./image/tabcontrol_mini.png) 


- **右键关闭菜单项** 

![tabcontrol_mini.png](./image/closeoption.png)

- **子页签配置操作**

页签2配置了操作，右边出现配置的功能按钮，监听事件代码见下方*<u>插件API</u>*
![tabmenu.png](./image/tabmenu.gif)

- **和向导控件组合使用**

![step.png](./image/step.png)

## 属性说明
属性名 | 属性ID | 类型 | 默认值 | 说明 
---- | :----: | :----: | :----: | ---- 
页签风格 | `tabStyle` | number | `3` | 切换页签风格
隐藏导航栏 | `hideNavigation` | boolean | `false` | 将导航栏隐藏  
关闭所有 | `canCloseAll` | boolean | `true` | 设置是否可以关闭所有可关闭的页签
滑动页签 | `canSlide` | boolean | `false` | 设置页签打开过多时可滑动显示页签，*<u>`简洁` 风格无该属性</u>*
全屏显示 | `canFull` | boolean | `false` | 是否允许页签全屏显示，*<u>只有`默认`风格有该属性</u>*
首页签 | `homeTab` | boolean | `false` | 设置当前页签是否为主页签，<u>*子页签有该属性*</u>
页签操作 | `tabPageOperationItems` | string |  | 为当前页签配置框架提供的操作，<u>*子页签有该属性*</u>

?> 控件通用属性[请参考](advanced/controlProperties/)

## 插件示例

- **插件中监听标签切换事件**

```java
public class TestPlugin extends AbstractFormPlugin implements TabSelectListener {
  public void registerListener (EventObject e){
    // 页签添加监听事件
    Tab tab = this.getView().getControl("页签控件标识");
    tab.addTabSelectListener(this);
  }
  
  // 重写tabSelected方法
  public void tabSelectd(TabSelectEvent event){
    // 点击页签的key
    String subTabKey = event.getTabKey();
    // do something...
  }
}
```  
- **插件中监听子页签配置的操作**

```java
public class TestPlugin extends AbstractFormPlugin implements ItemClickListener {
  public void registerListener (EventObject e) {
    // 页签添加监听事件
    Tab tab = this.getView().getControl("页签控件标识");
    tab.addItemClickListener(this);
  }
  
  public void beforItemClick(BeforeItemClickEvent event) {
    // 点击item之前触发
  }
   
  public void itemClick(ItemClickEvent event) {
    // 点击item时触发
  }
}
```
- **激活某个子页签**

```java
Tab tab = this.getView().getControl("页签标识");
tab.avtiveTab("子页签标识");
```  

- **隐藏和显示某个子页签**

![visiable.gif](./image/visiable.gif)

```java
// 隐藏某个页签
this.getView().setVisible(false, "子页签标识");

// 显示某个页签
this.getView().setVisible(true, "子页签标识");
```  
