package kd.bos.business.unittest.plugin.webofficedemo;

import kd.bos.cache.CacheFactory;
import kd.bos.cache.TempFileCache;
import kd.bos.context.RequestContext;
import kd.bos.data.BusinessDataWriter;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.earlywarn.kit.StringUtil;
import kd.bos.fileservice.FileItem;
import kd.bos.fileservice.FileService;
import kd.bos.fileservice.FileServiceFactory;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.Toolbar;
import kd.bos.form.control.WebOffice;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.control.events.UploadEvent;
import kd.bos.form.control.events.UploadListener;
import kd.bos.form.control.events.webOffice.WebOfficeDataListener;
import kd.bos.form.control.events.webOffice.event.WebOfficeBookmarkCheckEvent;
import kd.bos.form.control.events.webOffice.event.WebOfficeBookmarkEvent;
import kd.bos.form.control.events.webOffice.event.WebOfficeBookmarkValueEvent;
import kd.bos.form.control.events.webOffice.event.WebOfficeCheckboxEvent;
import kd.bos.form.control.events.webOffice.event.WebOfficeDocChangeEvent;
import kd.bos.form.control.events.webOffice.event.WebOfficeIsFieldReviseEvent;
import kd.bos.form.control.events.webOffice.event.WebOfficeOpenStatusEvent;
import kd.bos.form.control.model.WebOfficeDocProtectionType;
import kd.bos.form.control.model.WebOfficeMark;
import kd.bos.form.control.model.WebOfficeTrackingType;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.url.UrlService;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.UUID;

/**
 * @author rd_jihui_luo
 * @date 2020/2/26
 */
public class DocDetailPlugIn extends AbstractFormPlugin implements UploadListener{

    @Override
    public void afterCreateNewData(EventObject e) {
        FormShowParameter param = getView().getFormShowParameter();
        Object tplId = param.getCustomParam("tplId");
        if(tplId!=null){
        	initDocGrid(tplId);
        }
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
       
        FormShowParameter param = getView().getFormShowParameter();
        Object tplId = param.getCustomParam("tplId");
        //获取模板信息
        DynamicObject tpl = BusinessDataServiceHelper.loadSingle(tplId,"test_wb_doc_templete");
        DynamicObjectCollection perms = tpl.getDynamicObjectCollection("entity_edit_perm");
        String userId = RequestContext.get().getUserId();
        List<String> marks = new ArrayList<>();
        for (DynamicObject perm: perms) {
            DynamicObject user = (DynamicObject) perm.get("edit_user");
            String id;
            if(user!=null){     
            	id = user.getPkValue().toString();
                if(userId.equals(id)){
                    marks.add(perm.getString("bookmark"));
                }
            }
        }
        //发送打开指令 
        String url = tpl.getString("current_doc_url");
        if(StringUtils.isNotBlank(url)){
            url = UrlService.getAttachmentFullUrl(url);
            WebOffice webOffice = getControl("webofficeap");                
            webOffice.open(url);
            if(!marks.isEmpty()){
                //webOffice.limitDoc("webofficeap");
                webOffice.setEditableAreas(marks,"webofficeap");
                //webOffice.setDocUnLimitByRole(marks);
            }else{
                for (DynamicObject perm: perms) {
                    DynamicObject user = (DynamicObject) perm.get("edit_user");
                    String id = user.getPkValue().toString();
                    if(!userId.equals(id)){
                    	webOffice.deleteSelectedAreaEditBookmark(perm.getString("bookmark"));
                    }
                   
                }
            }
        }
    }
    private void initDocGrid(Object id) {
        DynamicObject object = BusinessDataServiceHelper.loadSingle(id,"test_wb_doc_templete");
        DynamicObjectCollection list = object.getDynamicObjectCollection("entry_variable");
        if(null != list && !list.isEmpty()){
            getModel().batchCreateNewEntryRow("entryentity_doc",list.size());
            for(int i = 0 ; i < list.size(); i++){
                DynamicObject obj = list.get(i);
                String name = obj.getString("variable_name");
                String markKey = obj.getString("variable_key");
                String markValue = obj.getString("variable_value");
                String varaible = obj.getString("variable_map");

                getModel().setValue("doc_name",name,i);
                getModel().setValue("doc_variable", varaible, i);
                getModel().setValue("doc_key", markKey, i);
                getModel().setValue("doc_value", markValue, i);
                getModel().setValue("doc_del","删除书签", i);
                getModel().setValue("doc_location","定位书签", i);
            }
        }

        DynamicObjectCollection perms = object.getDynamicObjectCollection("entity_edit_perm");
        if(null != perms && !perms.isEmpty()){
            getModel().batchCreateNewEntryRow("entity_edit",perms.size());
            for(int i = 0 ; i < perms.size(); i++){
                DynamicObject obj = perms.get(i);
                DynamicObject editor = obj.getDynamicObject("edit_user");
                String markValue = obj.getString("bookmark");

                getModel().setValue("editor",editor,i);
                getModel().setValue("edit_bookmark", markValue, i);
                getModel().setValue("edit_operation","删除书签", i);
            }
        }
    }
    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);

        Toolbar toolbar = getControl("toolbarap");
        toolbar.addItemClickListener(this);
        WebOffice webOffice = getControl("webofficeap");
        webOffice.addUploadListener(this);
    }
    @Override
    public void afterUpload(UploadEvent evt) {
    	// TODO Auto-generated method stub
    	UploadListener.super.afterUpload(evt);
		Object[] urls = evt.getUrls();
	    if(null == urls || urls.length == 0){
	        getView().showErrorNotification("请上传文档");
	        return;
	    }
	
	    Object urlData = urls[0];
	    String filePath = "";
	    String fileName = "";
	    String url = "";
        if(urlData instanceof String){
            filePath = urlData.toString();
            fileName = filePath.substring(filePath.lastIndexOf("/"));
            String[] files = fileName.split("\\.");
            String tFileName = files.length == 2 ? files[0] : UUID.randomUUID().toString();
            try {
            	tFileName = URLEncoder.encode(tFileName, "utf-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
            
            String tExtend = files.length == 2 ? files[1] : "doc";
            fileName = tFileName + "." + tExtend;
            
            url = filePath;
        }else if(urlData instanceof Map){
            filePath = ((Map<String,Object>) urlData).get("url").toString();
            fileName = UUID.randomUUID().toString();
            url = filePath;
        }
        
        if(url.contains("tempfile")){
            TempFileCache cache = CacheFactory.getCommonCacheFactory().getTempFileCache();
            InputStream in = cache.getInputStream(url);
            FileService service = FileServiceFactory.getAttachmentFileService();
            String suffix = ".doc"; 
            this.getPageCache().remove("opentype");
            String path = "/webofficedemo_" + System.currentTimeMillis() + "/" + fileName + suffix;
            try {
				fileName = URLEncoder.encode(fileName, "utf-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
            fileName = fileName + suffix;
            FileItem fileItem = new FileItem(fileName, path, in);
            fileItem.setCreateNewFileWhenExists(true);
            String turl = service.upload(fileItem);
            cache.remove(url);
            url = turl;
        }
        String key = evt.getCallbackKey(); 
        if("webofficeap".equals(key)){
        	DynamicObjectCollection bookMarks = getModel().getEntryEntity("entryentity_doc");
            FormShowParameter param = getView().getFormShowParameter();
            Long tplId = param.getCustomParam("tplId");
            DynamicObject object = BusinessDataServiceHelper.loadSingle(tplId,"test_wb_doc_templete");
            //更新filename
            object.set("current_doc_url", url);
            object.set("current_doc_name", fileName);

            //添加版本
            DynamicObjectCollection versions = object.getDynamicObjectCollection("entry_version");
            int versionCode = versions == null || versions.isEmpty() ? 1 : versions.get(versions.size()-1).getInt("version") + 1;
            DynamicObject version = new DynamicObject(versions.getDynamicObjectType());
            version.set("version_name", fileName);
            version.set("version_url", url);
            version.set("version", versionCode);
            version.set("seq",versionCode);
            versions.add(version);
            //更新书签
            DynamicObjectCollection list = object.getDynamicObjectCollection("entry_variable");
            list.clear();
            int index = 1;
            for(DynamicObject mark : bookMarks){
                DynamicObject obj = new DynamicObject(list.getDynamicObjectType());
                obj.set("variable_name", mark.get("doc_name"));
                obj.set("variable_key",mark.get("doc_key"));
                obj.set("variable_value",mark.get("doc_value"));
                obj.set("variable_map",mark.get("doc_variable"));
                obj.set("seq",index++);
                list.add(obj);
            }

            //更新权限书签
            DynamicObjectCollection permMarks = getModel().getEntryEntity("entity_edit");
            DynamicObjectCollection perms = object.getDynamicObjectCollection("entity_edit_perm");
            perms.clear();
            index = 1;
            for(DynamicObject mark : permMarks){
                DynamicObject obj = new DynamicObject(perms.getDynamicObjectType());
                obj.set("edit_user", mark.get("editor"));
                obj.set("bookmark",mark.get("edit_bookmark"));
                obj.set("seq",index++);
                WebOffice webOffice = getControl("webofficeap");
                webOffice.setSelectedAreaEditBookmark((String) mark.get("edit_bookmark"));
                perms.add(obj);
            }
            //sBusinessDataWriter.save(versions.getDynamicObjectType(), versions.toArray());
            BusinessDataWriter.save(object.getDynamicObjectType(), new Object[]{object});
            getView().showSuccessNotification("保存成功");
        }
    }
    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        switch(evt.getItemKey()){
            case "btn_replace": replaceDoc();break;
            case "save": save();break;
        }
    }
    private void save() {
        WebOffice webOffice = getControl("webofficeap");  
        webOffice.save("合同编号");
    }
    private void replaceDoc() {
        FormShowParameter param = getView().getFormShowParameter();
        Long id = param.getCustomParam("id");
        Long tplId = param.getCustomParam("tplId");

        //获取模板信息
        DynamicObject tpl = BusinessDataServiceHelper.loadSingle(tplId,"test_wb_doc_templete");
        DynamicObjectCollection tplVariables = tpl.getDynamicObjectCollection("entry_variable");

        //获取当前数据
        DynamicObject doc = BusinessDataServiceHelper.loadSingle(id,"test_wb_doc");

        //组装变量
        List<WebOfficeMark> marks = new ArrayList<>();
        for (DynamicObject variable : tplVariables) {
            String map = variable.getString("variable_map");
            String key = variable.getString("variable_key");          
            if(map!=""){
                String value = StringUtil.toSafeString(doc.get(map));
                marks.add(new WebOfficeMark(key,value));
            }


        }

        //发送替换书签指令
        if(!marks.isEmpty()){
            WebOffice webOffice = getControl("webofficeap");
            webOffice.modifyMarks(marks);
        }
    }

    public static FormShowParameter show(Long id, Long tplId) {
        FormShowParameter param = new FormShowParameter();
        param.setFormId("test_wb_doc_detail");
        param.getOpenStyle().setShowType(ShowType.Modal);
        param.setCustomParam("id",id);
        param.setCustomParam("tplId",tplId);
        return param;
    }



}
